/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <Ogre.h>
#include "GameActor.h"
#include "PHYSICS/body.h"
#include "PHYSICS/shape.h"

//! Clase que representa los ataques del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Modela los ataques dentro del juego, ya sean cuerpo a cuerpo o a distancia.
 *  
 *  Hereda de la clase principal, GameActor, y funciona como si fuera una piedra,
 *  avanza hacia adelante hasta su límite de distancia, si pega contra algo, le hace daño.
 * 
 *  La única diferencia entre cuerpo a cuerpo y a distancia es el límite de alcance.
 */

class Attack : public GameActor
{
public:

	enum ATTACK_TYPE {
            NONE, //por defecto
            MELEE,
            RANGE,
            AREA
	};

	Attack(int id, ATTACK_TYPE type, float range, float damage, int manaCost, Ogre::SceneNode* proyectil, Shape* shape, Ogre::Vector3 direction);
	~Attack(void);

	ATTACK_TYPE getAttackType();
	float getRange();
	float getDamage();
	int getManaCost();
	void executeAttack();
	void finalizeAttack();
	bool isActive();

	void setOwner(GameActor* owner);
	GameActor* getOwner();

	void update(float elapsedTime);
        
        void advance(float vy);

private:

	GameActor* mOwner;

	ATTACK_TYPE mType;

	float mRange;
	float mDamage;
	bool mIsActive;
	int mManaCost;
	Ogre::Vector3 mDirection;
        
        float mTravesedDistance;

};
