#include "EstadoJugando.h"
#include "Evt_Mover.h"
#include "Evt_Camara.h"
#include "Evt_Ver.h"
#include "Evt_Visto.h"
#include "Evt_Atacar.h"
#include "Evt_Atacado.h"
#include "Evt_AplicarAtaque.h"
#include "Evt_Colision.h"
#include "Evt_QuitarJuego.h"
#include "Attack.h"
#include "IA/AIViewDeer.h"
#include "IA/AIViewWolf.h"
#include "IA/AIViewDragon.h"

#include "AttackView.h"
#include "GestorSonido.h"
#include "Evt_Sonido.h"

#include <typeinfo>
#include "XML/pugixml.hpp"
#include <iostream>
#include <vector>
#include <algorithm>
#include <ctime>


using namespace Ogre;
using namespace GardeGames;

EstadoJugando::EstadoJugando(void)
: GameState("EstadoJugando")
{
    
}

EstadoJugando::~EstadoJugando(void)
{
	if (mAccionSalir) {
		delete mAccionSalir;
	}
	if (mAccionAlante) {
		delete mAccionAlante;
	}
	if (mAccionAtras) {
		delete mAccionAtras;
	}
	if (mAccionIzda) {
		delete mAccionIzda;
	}
	if (mAccionDcha) {
		delete mAccionDcha;
	}
	if (mAccionRatonMovido) {
		delete mAccionRatonMovido;
	}
	if (mHumanView) {
		delete mHumanView;
	}
        
        if (mGestorOverlay) {
            mGestorOverlay->destroyAll();
            delete mGestorOverlay;
            mGestorOverlay=0;
        }
}

void EstadoJugando::inicializarCiervo(Ogre::Vector3 position,double maxSpeed)
{
    GameActor* aux;
    Creature* mCiervo;
    aux=GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Deer,Ogre::Vector3(100,5,-50));
    mCiervo=dynamic_cast<Creature*>(aux);
    mCiervo->setMaxSpeed(maxSpeed);
    mCollisionManager->addBody(mCiervo->getBody());
    Ogre::Vector3 finalPos=mCiervo->getPosition();
    mCiervo->setHeight(mGestor->getHeightAtPosition(finalPos));
    mAIView1 = new AIViewDeer(mCiervo,mGestor->createLifeBar(mCiervo));
    mGameViews.push_back(mAIView1);
    
}

void EstadoJugando::inicializarDragon(Ogre::Vector3 position,double maxSpeed)
{
    GameActor* aux;
    Creature* mDragon;
    aux=GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Dragon,position);
    mDragon=dynamic_cast<Creature*>(aux);
    mDragon->setMaxSpeed(maxSpeed);
    mCollisionManager->addBody(mDragon->getBody());
    Ogre::Vector3 finalPos=mDragon->getPosition();
    mDragon->setHeight(mGestor->getHeightAtPosition(finalPos));
    mAIView3 = new AIViewDragon(mDragon,mGestor->createLifeBar(mDragon));
    mGameViews.push_back(mAIView3);
    
}



void EstadoJugando::inicializarLobo(Ogre::Vector3 position,double maxSpeed)
{
    GameActor* aux;
    Creature* mLobo;
    aux=GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Wolf,position);
    mLobo=dynamic_cast<Creature*>(aux);
    mLobo->setMaxSpeed(maxSpeed);
    mCollisionManager->addBody(mLobo->getBody());
    Ogre::Vector3 finalPos=mLobo->getPosition();
    mLobo->setHeight(mGestor->getHeightAtPosition(finalPos));
    mAIView2 = new AIViewWolf(mLobo,mGestor->createLifeBar(mLobo));
    mGameViews.push_back(mAIView2);
    
}
void EstadoJugando::loadActorsFromXML(const Ogre::String& fileName)
{
   pugi::xml_document doc;
    std::vector<int> a, b, c;
    std::vector<Ogre::Vector3> vertex;
    
    // Abrimos el fichero XML
    if (!doc.load_file(fileName.c_str())) {
        cerr << "EstadoJugando::ActorsFromXML(): error al leer el fichero " << fileName << endl;
        exit(1);
    }
    
    //Buscamos los nodos principales
    pugi::xml_node actorNode = doc.child("actor");
    pugi::xml_node wolfNode = actorNode.child("wolf");
    pugi::xml_node deerNode = actorNode.child("deer");
      pugi::xml_node dragonNode = actorNode.child("dragon");
    
    //Creamos los lobos
    
    pugi::xml_node velocityNode=wolfNode.child("maxSpeed");
    double maxSpeed=velocityNode.attribute("speed").as_double();
    
    pugi::xml_node positionsNode = wolfNode.child("position");
    
    // Tomamos las caras
    for (pugi::xml_node posNode = positionsNode.child("pos"); posNode; posNode = posNode.next_sibling()) {
        int x = posNode.attribute("x").as_double();
        int y = posNode.attribute("y").as_double();
        int z = posNode.attribute("z").as_double();
        
        inicializarLobo(Ogre::Vector3(x,y,z),maxSpeed);
    }
    
    //creamos los ciervos
    
    velocityNode=deerNode.child("maxSpeed");
    maxSpeed=velocityNode.attribute("speed").as_double();
    
    positionsNode = deerNode.child("position");
    
    // Tomamos las caras
    for (pugi::xml_node posNode = positionsNode.child("pos"); posNode; posNode = posNode.next_sibling()) {
        int x = posNode.attribute("x").as_double();
        int y = posNode.attribute("y").as_double();
        int z = posNode.attribute("z").as_double();
        
        inicializarCiervo(Ogre::Vector3(x,y,z),maxSpeed);
    }
    
    //creamos los dragones
    
    velocityNode=dragonNode.child("maxSpeed");
    maxSpeed=velocityNode.attribute("speed").as_double();
    
    positionsNode = dragonNode.child("position");
    
    // creamos los dragones
    for (pugi::xml_node posNode = positionsNode.child("pos"); posNode; posNode = posNode.next_sibling()) {
        int x = posNode.attribute("x").as_double();
        int y = posNode.attribute("y").as_double();
        int z = posNode.attribute("z").as_double();
        
        inicializarDragon(Ogre::Vector3(x,y,z),maxSpeed);
    }
    
    

}
void EstadoJugando::start(GardeGames::InputManager *inputManager)
{
	//GestorEscenario::getSingletonPtr()->loadEscenario("terreno de prueba.agm");
	GestorEscenario::getSingletonPtr()->createNewEscenario("terreno de prueba.agm","terreno de prueba.agm");
        //GestorEscenario::getSingletonPtr()->addElement(GestorEscenario::OGRO,Ogre::Vector3(100,50,100));
        //GestorEscenario::getSingletonPtr()->addElement(GestorEscenario::ROBOT,Ogre::Vector3(500,50,100));
        //GestorEscenario::getSingletonPtr()->addElement(GestorEscenario::NINJA,Ogre::Vector3(300,0,500));

	//-------------creacion del jugador--------------------
	GameActor* aux=GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Player,Ogre::Vector3(100,5,0));
	mPj=dynamic_cast<Creature*>(aux);
        mCollisionManager->addBody(mPj->getBody());
        //Creature* auxCiervo=dynamic_cast<Creature*>(GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Deer,Ogre::Vector3(100,5,0)));
        //Creature* auxLobo=dynamic_cast<Creature*>(GestorEscenario::getSingletonPtr()->createGameActor(GameActor::Wolf,Ogre::Vector3(100,5,0)));
        
        Ogre::Vector3 finalPos=mPj->getPosition();
        mPj->setHeight(mGestor->getHeightAtPosition(finalPos));
	//-----------------------------------------------------
    
	
        loadActorsFromXML("level1.xml");

/*	aux=mGameActorManager->createGameActor(GameActor::House,Ogre::Vector3(190,271,0));
	mCasa1=dynamic_cast<StaticActor*>(aux);

	aux=mGameActorManager->createGameActor(GameActor::House,Ogre::Vector3(190,271,50));
	mCasa2=dynamic_cast<StaticActor*>(aux);

	aux=mGameActorManager->createGameActor(GameActor::House,Ogre::Vector3(190,276,150));
	mCasa3=dynamic_cast<StaticActor*>(aux);
*/
	//ParticleSystem* ps = mGestor->getSceneManager()->createParticleSystem("PurpleFountain", "Examples/PurpleFountain");
	//ParticleSystem* ps = mGestor->getSceneManager()->createParticleSystem("rayo", "FireRain");
	//ParticleSystem* ps = mGestor->getSceneManager()->createParticleSystem("rayo", "PEExamples/blast");

	//-----------------------------------------
	mRenderWindow->addViewport(mCamera);
	Ogre::ColourValue fadeColour(0.9, 0.9, 0.9);
	mRenderWindow->getViewport(0)->setBackgroundColour(fadeColour);

	inputManager->mapToKey(mAccionSalir,OIS::KC_ESCAPE);
	inputManager->mapToKey(mAccionAlante,OIS::KC_W);
	inputManager->mapToKey(mAccionAtras,OIS::KC_S);
	inputManager->mapToKey(mAccionIzda,OIS::KC_A);
	inputManager->mapToKey(mAccionDcha,OIS::KC_D);	
        
	inputManager->mapToKey(mAccionVolumeUp,OIS::KC_INSERT);
	inputManager->mapToKey(mAccionVolumeDown,OIS::KC_DELETE);	

	inputManager->mapToMouse(mAccionRatonMovido,GardeGames::InputManager::MOUSE_MOVE_DOWN);
	inputManager->mapToMouse(mAccionRatonMovido,GardeGames::InputManager::MOUSE_MOVE_UP);
	inputManager->mapToMouse(mAccionRatonMovido,GardeGames::InputManager::MOUSE_MOVE_LEFT);
	inputManager->mapToMouse(mAccionRatonMovido,GardeGames::InputManager::MOUSE_MOVE_RIGHT);

	inputManager->mapToMouse(mAccionGolpea,GardeGames::InputManager::MOUSE_BUTTON_LEFT);
	inputManager->mapToMouse(mAccionLanzaRayo,GardeGames::InputManager::MOUSE_BUTTON_RIGHT);
        
        inputManager->mapToKey(mAccionTransElfa,OIS::KC_1);
	inputManager->mapToKey(mAccionTransCiervo,OIS::KC_2);
	inputManager->mapToKey(mAccionTransLobo,OIS::KC_3);

	this->mInputManager=inputManager;

	this->siguienteEstado="";

	

	//inicializar eventos, vistas y listeners
	GardeGames::EventListenerPtr listener ( this );
	
	GardeGames::IEventManager *evtMgr=GardeGames::IEventManager::Get();

       
	evtMgr->addListener( listener, GardeGames::EventType(Evt_Mover::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_Camara::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_Ver::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_Atacar::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_ComenzarAtaque::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_AplicarAtaque::gkName) );
	evtMgr->addListener( listener, GardeGames::EventType(Evt_Colision::gkName) );        
	evtMgr->addListener( listener, GardeGames::EventType(Evt_QuitarJuego::gkName) );
	
	//mGestor->addElement(GestorEscenario::NINJA,Ogre::Vector3(0,350,0))        
        configurarOverlay();
	mHumanView=new HumanView(mPj,
                                                        mLifeBar,
							mAccionAlante,
							mAccionAtras,
							mAccionIzda,
							mAccionDcha,
							mAccionRatonMovido,
							mAccionGolpea,
							mAccionLanzaRayo,
                                                        mAccionTransElfa,
                                                        mAccionTransLobo,
                                                        mAccionTransCiervo,
							mCamera);
	
        
        mGameViews.push_back(mHumanView);
    
	/*
	mAIView1=new AIView(mGestor->addElement(GestorEscenario::ROBOT,Ogre::Vector3(50,350,0)));
	mAIView2=new AIView(mGestor->addElement(GestorEscenario::ROBOT,Ogre::Vector3(0,350,100)));
	mAIView3=new AIView(mGestor->addElement(GestorEscenario::ROBOT,Ogre::Vector3(100,350,0)));
	mAIView4=new AIView(mGestor->addElement(GestorEscenario::ROBOT,Ogre::Vector3(0,350,50)));
	mAIView5=new AIView(mGestor->addElement(GestorEscenario::ROBOT,Ogre::Vector3(100,350,100)));
	*/
	mCajaTL=mGestor->getSceneManager()->getRootSceneNode()->createChildSceneNode("tl",Ogre::Vector3::ZERO);//mGameActorManager->mRenderSystem->createNode();
	//mCajaTL->setPosition(Ogre::Vector3(0,400,0));
	//mCajaTL->createAndAttachEntity("cube.1m.mesh");
        // Creamos la entidad
        Ogre::Entity *_entityTL = mGestor->getSceneManager()->createEntity("tl", "cube.1m.mesh");
        // Adjuntamos la entidad al nodo
        mCajaTL->attachObject(_entityTL);

	mCajaTR=mGestor->getSceneManager()->getRootSceneNode()->createChildSceneNode("tr",Ogre::Vector3::ZERO);//mGameActorManager->mRenderSystem->createNode();
	//mCajaTR->setPosition(Ogre::Vector3(0,400,0));
	//mCajaTR->createAndAttachEntity("cube.1m.mesh");
        Ogre::Entity *_entityTR = mGestor->getSceneManager()->createEntity("tr", "cube.1m.mesh");
        // Adjuntamos la entidad al nodo
        mCajaTR->attachObject(_entityTR);
	
	mCajaBL=mGestor->getSceneManager()->getRootSceneNode()->createChildSceneNode("bl",Ogre::Vector3::ZERO);//mGameActorManager->mRenderSystem->createNode();
	//mCajaBL->setPosition(Ogre::Vector3(0,400,0));
	//mCajaBL->createAndAttachEntity("cube.1m.mesh");
        Ogre::Entity *_entityBL = mGestor->getSceneManager()->createEntity("bl", "cube.1m.mesh");
        // Adjuntamos la entidad al nodo
        mCajaBL->attachObject(_entityBL);
	
	mCajaBR=mGestor->getSceneManager()->getRootSceneNode()->createChildSceneNode("br",Ogre::Vector3::ZERO);//mGameActorManager->mRenderSystem->createNode();
	//mCajaBR->setPosition(Ogre::Vector3(0,400,0));
	//mCajaBR->createAndAttachEntity("cube.1m.mesh");
        Ogre::Entity *_entityBR = mGestor->getSceneManager()->createEntity("br", "cube.1m.mesh");
        // Adjuntamos la entidad al nodo
        mCajaBR->attachObject(_entityBR);

	mElapsedTime=0;        
}

void EstadoJugando::configurarOverlay() {
    // Tomamos el overlay
    mGestorOverlay = Ogre::OverlayManager::getSingletonPtr();
    
    // Recuperamos los elementos del overlay
    mOverlay = mGestorOverlay->getByName("OverlayInfo"); 
    mLifeBar = static_cast<Ogre::PanelOverlayElement*>(mGestorOverlay->getOverlayElement("lifeBar"));
    mBackBar = static_cast<Ogre::PanelOverlayElement*>(mGestorOverlay->getOverlayElement("backBar"));
    
    if (mOverlay->isInitialised())
        cout << "Overlay inicializado" << endl;
    else
        cout << "Overlay no inicializado" << endl;
    
    // Mostramos el overlay
    mOverlay->show();
}

void EstadoJugando::stop()
{
	mRenderWindow->removeAllViewports();
        //mSong->stop();
}

void EstadoJugando::update(long elapsedTime)
{
	if (mGestor->isLoading()) {
		return;
	}

	if (mAccionSalir->isPressed()) {
		this->siguienteEstado=GardeGames::GameStateManager::exitGame();
		return;
	}
        if (mAccionVolumeUp->isPressed()) {
            GestorSonido::getSingletonPtr()->volumeUp();
	}
        if (mAccionVolumeDown->isPressed()) {
            GestorSonido::getSingletonPtr()->volumeDown();
	}
	mElapsedTime=(float)elapsedTime/1000;

	//actualizar fisicas
//	mGameActorManager->mWorld->advance((float)elapsedTime/1000);

        
        //reajustamos la altura si es necesario
        Ogre::Vector3 origen=mHumanView->getCamPosition();	
        float alturaSuelo=mGestor->getHeightAtPosition(origen);      
        if (alturaSuelo>=origen.y) {        
            //calculamos un nuevo giro en X para que la camara quede por encima del suelo        
            Ogre::Vector3 posFinal = Ogre::Vector3(origen.x,alturaSuelo+2,origen.z)-mHumanView->getPivotPosition();                          
            Ogre::Radian radianAngulo = posFinal.angleBetween(origen-mHumanView->getPivotPosition());        
            // forzamos un nuevo giero en X        
            mHumanView->updateCameraGoal(0,-radianAngulo.valueAngleUnits(),0);	
            //printf("Posicion corregida en %f grados\n",-radianAngulo.valueAngleUnits());
        }
        
        /*
	mHumanView->VOnUpdate(elapsedTime);        
	mAIView1->VOnUpdate(elapsedTime);
        mAIView2->VOnUpdate(elapsedTime);
        */
        //hago esto porque no me deja declarar un objeto de tipo GardeGames::IGameView y recorrer el vector con el...
        for (int i=0; i< mGameViews.size(); i++) {
            switch (mGameViews.at(i)->VGetType()) {
                case GardeGames::IGameView::GameView_Human :
                    HumanView *hv=dynamic_cast<HumanView*>(mGameViews.at(i));
                    hv->VOnUpdate(elapsedTime);
                    mPj=hv->getPlayer();
                    break;
                case GardeGames::IGameView::GameView_AI :
                    AIView *aiv=dynamic_cast<AIView*>(mGameViews.at(i));
                    aiv->VOnUpdate(elapsedTime);
                    break;
                case GardeGames::IGameView::GameView_Other :
                    AttackView *av=dynamic_cast<AttackView*>(mGameViews.at(i));
                    av->VOnUpdate(elapsedTime);
                    break;
            }
        }
        mCollisionManager->checkCollisions();
        GestorSonido::getSingletonPtr()->update(elapsedTime);
        //
        
		
/*
	mAIView1->VOnUpdate(elapsedTime);
	mAIView2->VOnUpdate(elapsedTime);
	mAIView3->VOnUpdate(elapsedTime);
	mAIView4->VOnUpdate(elapsedTime);
	mAIView5->VOnUpdate(elapsedTime);
*/
}

void EstadoJugando::loadResources() {
	
	mRoot         = Root::getSingletonPtr();
	mGestor		  = GestorEscenario::getSingletonPtr();
	mSceneMgr	  = mGestor->getSceneManager();
	mRenderWindow = mRoot->getAutoCreatedWindow();
	mAlturaCamara = 25;
//	mGameActorManager = GameActorManager::getSingleton();


	//creacion de la camara
	mCamera = mGestor->getCamera();
	mCamera->setNearClipDistance(0.1);
	if (mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_INFINITE_FAR_PLANE))
	{
		mCamera->setFarClipDistance(0);   // enable infinite far clip distance if we can
	}		
	else {
		mCamera->setFarClipDistance(50000);
	}
	mCamera->setAspectRatio((Ogre::Real)16/9);

	//creacion de las acciones del usuario
	mAccionSalir = new GameAction("salir",GameAction::NORMAL);
	mAccionAlante = new GameAction("alante",GameAction::NORMAL);
	mAccionAtras = new GameAction("atras",GameAction::NORMAL);
	mAccionIzda = new GameAction("izquierda",GameAction::NORMAL);
	mAccionDcha = new GameAction("derecha",GameAction::NORMAL);	
	mAccionRatonMovido = new GameAction("mover",GameAction::NORMAL);
	mAccionGolpea = new GameAction("golpe",GameAction::DETECT_INITAL_PRESS_ONLY);
	mAccionLanzaRayo = new GameAction("rayo",GameAction::DETECT_INITAL_PRESS_ONLY);
	mAccionVolumeUp = new GameAction("vUp",GameAction::NORMAL);	
	mAccionVolumeDown = new GameAction("vDown",GameAction::NORMAL);
	mAccionTransElfa = new GameAction("transElfa",GameAction::DETECT_INITAL_PRESS_ONLY);
	mAccionTransCiervo = new GameAction("transCiervo",GameAction::DETECT_INITAL_PRESS_ONLY);
	mAccionTransLobo = new GameAction("transLobo",GameAction::DETECT_INITAL_PRESS_ONLY);
        
        CollisionManager::CollisionCallback callback;
        
        mCollisionManager=new CollisionManager();
        
        callback = boost::bind(&EstadoJugando::HandlePhysics_Body_Body_BeginEvent, this, _1, _2);
/*         
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Deer, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::House, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Player, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Wolf, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Wolf, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Player, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::House, callback, CollisionManager::BEGINCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Player, GameActor::House, callback, CollisionManager::BEGINCOLLISION);
*/       
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Deer, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::House, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Player, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Wolf, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Dragon, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Wolf, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Player, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::House, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Dragon, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Player, GameActor::House, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Player, GameActor::Player, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Player, GameActor::Dragon, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Dragon, GameActor::Dragon, callback, CollisionManager::COLLIDING);
        
        
        
        
        
        callback = boost::bind(&EstadoJugando::HandlePhysics_Attack_Body_BeginEvent, this, _1, _2);
        mCollisionManager->addCollisionCallback(GameActor::Attack, GameActor::Deer, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Attack, GameActor::House, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Attack, GameActor::Wolf, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Attack, GameActor::Player, callback, CollisionManager::COLLIDING);
        mCollisionManager->addCollisionCallback(GameActor::Attack, GameActor::Dragon, callback, CollisionManager::COLLIDING);

/*        
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Deer, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::House, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Player, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Deer, GameActor::Wolf, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Wolf, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::Player, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Wolf, GameActor::House, callback, CollisionManager::ENDCOLLISION);
        mCollisionManager->addCollisionCallback(GameActor::Player, GameActor::House, callback, CollisionManager::ENDCOLLISION);
        
        callback = boost::bind(&EstadoJugando::HandlePhysicsDuringEvent, this, _1, _2);
        collisionManager->addCollisionCallback(GameObject::PLAYER, GameObject::SCENE, callback, CollisionManager::COLLIDING);
        
        callback = boost::bind(&EstadoJugando::HandlePhysicsEndEvent, this, _1, _2);
        collisionManager->addCollisionCallback(GameObject::PLAYER, GameObject::SCENE, callback, CollisionManager::ENDCOLLISION);
*/
        GestorSonido::getSingletonPtr()->setBackMusic("back_music.wav");
}



void EstadoJugando::HandlePhysics_Body_Body_BeginEvent(Body* bodyA, Body* bodyB) {
    GardeGames::IEventManager *evtMgr=GardeGames::IEventManager::Get();
    GameActor* ga1=bodyA->getGameActor();
    GameActor* ga2=bodyB->getGameActor();
    evtMgr->queueEvent(GardeGames::EventPtr( new Evt_Colision(ga1,ga2)));    
    printf("Lanzado evento colision!!!!\n\n");
}

void EstadoJugando::HandlePhysics_Attack_Body_BeginEvent(Body* bodyA, Body* bodyB) {    
    GardeGames::IEventManager *evtMgr=GardeGames::IEventManager::Get();
    GameActor* ga1=bodyA->getGameActor();
    GameActor* ga2=bodyB->getGameActor();
    if (!ga1 || !ga2) {
        return;
    }
    if (ga1->getType()==GameActor::Attack) {
        Attack* at=dynamic_cast<Attack*>(ga1);
        if (at->isActive() && at->getOwner()->getId()!=ga2->getId()) {
            evtMgr->queueEvent(GardeGames::EventPtr( new Evt_AplicarAtaque(at,ga2)));
        }
    }
    else if (ga2->getType()==GameActor::Attack) {
        Attack* at=dynamic_cast<Attack*>(ga2);
        if (at->isActive() && at->getOwner()->getId()!=ga1->getId()) {
            evtMgr->queueEvent(GardeGames::EventPtr( new Evt_AplicarAtaque(at,ga1)));
        }
    }
}
/*
void EstadoJugando::HandlePhysicsDuringEvent() {
    
}

void EstadoJugando::HandlePhysicsEndEvent() {
    
}
*/
bool EstadoJugando::HandleEvent( GardeGames::Event & evento ) {

	GardeGames::IEventManager *evtMgr=GardeGames::IEventManager::Get();

	if (stricmp(evento.getType().getStr(),Evt_Mover::gkName)==0) {
		Evt_Mover evt=dynamic_cast<Evt_Mover &>(evento);
		Ogre::Vector3 direccion=evt.mDesplazamiento*mElapsedTime;
                
                if (evt.mElemento->getType()!=GameActor::Attack) { //si no es un ataque es una criatura
                    Creature* cr=dynamic_cast<Creature*>(evt.mElemento);
                    cr->setState(Creature::RUN,true);
                    cr->applyVelocity(-direccion.x,direccion.z);
                    Ogre::Vector3 finalPos=evt.mElemento->getPosition();
                    cr->setHeight(mGestor->getHeightAtPosition(finalPos));
                    cr->synchronizeBody();
                }
                else {
                    Attack* at=dynamic_cast<Attack*>(evt.mElemento);
                    at->advance(direccion.z);
                    at->synchronizeBody();
                    Ogre::Vector3 posicionParticula=at->getPosition();
                    float alturaSuelo=mGestor->getHeightAtPosition(posicionParticula);
                    if (alturaSuelo>=posicionParticula.y) {
                        at->finalizeAttack();
                        mCollisionManager->removeBody(at->getBody());
                    }
                }
	}
        
  
	else if (stricmp(evento.getType().getStr(),Evt_Camara::gkName)==0) {
        
            mHumanView->updateCameraGoal(-mInputManager->getMouseRelativeX()/5,-mInputManager->getMouseRelativeY()/5,0);
            Ogre::Vector3 origen=mHumanView->getCamPosition();	
            float alturaSuelo=mGestor->getHeightAtPosition(origen);  
            //mHumanView->adjustCamHeight(alturaSuelo);
            //reajustamos la altura si es necesario
            /*Ogre::Vector3 origen=mHumanView->getCamPosition();	
            float alturaSuelo=mGestor->getHeightAtPosition(origen);      
            if (alturaSuelo>=origen.y) {        
                //calculamos un nuevo giro en X para que la camara quede por encima del suelo        
                Ogre::Vector3 posFinal = Ogre::Vector3(origen.x,alturaSuelo+5,origen.z);                          
                Ogre::Radian radianAngulo = posFinal.angleBetween(origen);        
                // forzamos un nuevo giero en X        
                mHumanView->updateCameraGoal(0,-radianAngulo.valueDegrees(),0);	
            }*/
	}
	else if (stricmp(evento.getType().getStr(),Evt_Ver::gkName)==0) {
		Evt_Ver evt=dynamic_cast<Evt_Ver &>(evento);
		Ogre::Vector3 position = evt.mCreature->getPosition();

		Ogre::Vector2 tl(/*position.x*/-evt.mMaxAmplitud/2,/*position.z*/15+evt.mAlcance);
		Ogre::Vector2 tlFinal = globalPosition(tl,evt.mCreature->getOrientation(),position);
		Ogre::Ray topLeft;
		topLeft.setOrigin(Ogre::Vector3(tlFinal.x,position.y+100,tlFinal.y));
		topLeft.setDirection(Ogre::Vector3::NEGATIVE_UNIT_Y);

		Ogre::Vector2 tr(/*position.x*/+evt.mMaxAmplitud/2,/*position.z*/15+evt.mAlcance);
		Ogre::Vector2 trFinal = globalPosition(tr,evt.mCreature->getOrientation(),position);
		Ogre::Ray topRight;
		topRight.setOrigin(Ogre::Vector3(trFinal.x,position.y+100,trFinal.y));
		topRight.setDirection(Ogre::Vector3::NEGATIVE_UNIT_Y);

		Ogre::Vector2 bl(/*position.x*/-5,/*position.z*/+15);
		Ogre::Vector2 blFinal = globalPosition(bl,evt.mCreature->getOrientation(),position);
		Ogre::Ray bottomLeft;
		bottomLeft.setOrigin(Ogre::Vector3(blFinal.x,position.y+100,blFinal.y));
		bottomLeft.setDirection(Ogre::Vector3::NEGATIVE_UNIT_Y);

		Ogre::Vector2 br(/*position.x*/+5,/*position.z*/+15);
		Ogre::Vector2 brFinal = globalPosition(br,evt.mCreature->getOrientation(),position);
		Ogre::Ray bottomRight;
		bottomRight.setOrigin(Ogre::Vector3(brFinal.x,position.y+100,brFinal.y));
		bottomRight.setDirection(Ogre::Vector3::NEGATIVE_UNIT_Y);

		Ogre::SceneQueryResult result = mGestor->getVolumeResult(topLeft,topRight,bottomLeft,bottomRight);

		if (!result.movables.empty()) { //si hay algo a la vista
			std::vector<GameActor*> actoresVistos;
			Ogre::SceneQueryResultMovableList::iterator iter;
			for (iter = result.movables.begin(); iter != result.movables.end(); ++iter){
				int index;
				try {
					index=atoi((*iter)->getName().c_str());
					if (index>0) { //cero no es nada
						actoresVistos.push_back(mGestor->getGameActorAt(index));
					}
				}
				catch (Exception &e) {
					//el indice no es correcto (se supone un terreno o similar)
				}
			}
			if (!actoresVistos.empty()) { //si hay criaturas mandamos evento
                            //printf("Vista criatura\n");
				evtMgr->queueEvent(GardeGames::EventPtr( new Evt_Visto(evt.mCreature,actoresVistos)));
			}
		}

		//prueba
		mCajaTL->setPosition(Ogre::Vector3(tlFinal.x,position.y+10,tlFinal.y));
		mCajaTR->setPosition(Ogre::Vector3(trFinal.x,position.y+10,trFinal.y));
		mCajaBL->setPosition(Ogre::Vector3(blFinal.x,position.y+10,blFinal.y));
		mCajaBR->setPosition(Ogre::Vector3(brFinal.x,position.y+10,brFinal.y));
	}	
	else if (stricmp(evento.getType().getStr(),Evt_Atacar::gkName)==0) {
		Evt_Atacar evt=dynamic_cast<Evt_Atacar &>(evento);
                if (evt.mType!=Attack::NONE) {                    
                    Attack* attack=mGestor->createAttack(evt.mType,evt.mAtacante,evt.mDirection,evt.mHeight);
                    attack->executeAttack();
                    mGameViews.push_back(new AttackView(attack));
                    mCollisionManager->addBody(attack->getBody());
                    //Sound3D* sonido = new Sound3D("fire.wav");                                      
                    //evtMgr->queueEvent(GardeGames::EventPtr( new Evt_Sonido(attack,Evt_Sonido::Attack,sonido,false)));
                }
	}
	else if (stricmp(evento.getType().getStr(),Evt_ComenzarAtaque::gkName)==0) {
		Evt_ComenzarAtaque evt=dynamic_cast<Evt_ComenzarAtaque &>(evento);
		//tratarComenzarAtaque(evt);

	}
        else if (stricmp(evento.getType().getStr(),Evt_QuitarJuego::gkName)==0) {
            Evt_QuitarJuego evt=dynamic_cast<Evt_QuitarJuego &>(evento);
            printf("Recibido Evento de quitar.\n");
            if (evt.mGameActor->getType()!=GameActor::Player) {
                if (evt.mGameActor->getType()!=GameActor::Attack) {
                    mGestor->destroyLifeBar(evt.mGameActor);
                    mCollisionManager->removeBody(evt.mGameActor->getBody());
                }
                else {
                    //quitar particulas
                }
            }
            else {                
                mCollisionManager->removeBody(evt.mGameActor->getBody());
            }
            
            mGestor->destroyGameActor(evt.mGameActor);
            printf("Actor destruido.\n");
            GardeGames::IGameView* vistaAQuitar=evt.mVista;
            printf("Vista a quitar: %d.\n",vistaAQuitar);
            //quitar la lista de manera jujana:
            GardeGames::IGameView* aux;                        
            std::vector<GardeGames::IGameView*> nuevaLista;
            printf("Vistas: %d.\n",mGameViews.size());
            int n=mGameViews.size();
            for (int i=0; i<n; i++) {
                aux=mGameViews.back();
                printf("Vista aux: %d.\n",aux);
                mGameViews.pop_back();
                if (aux!=vistaAQuitar) {
                    nuevaLista.push_back(aux);
                }
            }
            mGameViews=nuevaLista;
            //mGameViews.erase(mGameViews.begin()+);
            //delete vistaAQuitar;
            //vistaAQuitar=0;
            printf("Vista destruida.\n");
            printf("Vistas: %d.\n",mGameViews.size());
        }
	else if (stricmp(evento.getType().getStr(),Evt_AplicarAtaque::gkName)==0) {
		Evt_AplicarAtaque evt=dynamic_cast<Evt_AplicarAtaque &>(evento);
                if (!evt.mAtaque) {
                    return false;
                }
		evtMgr->queueEvent(GardeGames::EventPtr( new Evt_Atacado(evt.mAtaque->getOwner(),evt.mAtacado,evt.mAtaque->getDamage())));
                evt.mAtaque->finalizeAttack();
                mCollisionManager->removeBody(evt.mAtaque->getBody());
                //mGestor->destroyGameActor(evt.mAtaque);
	}
        else if (stricmp(evento.getType().getStr(),Evt_Colision::gkName)==0) {
		Evt_Colision evt=dynamic_cast<Evt_Colision &>(evento);
                printf("SE ALEJAN!!!!\n\n");
                //restauramos las posiciones (el que sea estatico no cambiara de posicion
                evt.mGameActorA->restorePosition();
                evt.mGameActorB->restorePosition();
                
                //los separamos un poco (moviendo solo a los que no son estaticos ni son el jugador)
                Ogre::Vector3 direction =  evt.mGameActorA->getPosition() - evt.mGameActorB->getPosition();
                direction.y=0;
                direction.normalise();
                direction=direction; //factor de alejamiento
                GameActor::ActorType tipoA=evt.mGameActorA->getType();
                GameActor::ActorType tipoB=evt.mGameActorB->getType();
                if (tipoA!=GameActor::House){
                    Creature* crA=dynamic_cast<Creature*>(evt.mGameActorA);
                    if  (crA!=mPj) {
                        crA->getNode()->setPosition(crA->getPosition() + direction*0.35);
                        //crA->applyVelocity(direction.x,direction.z);       
                        Ogre::Vector3 finalPosA=crA->getPosition();        
                        crA->setHeight(mGestor->getHeightAtPosition(finalPosA));     
                        crA->synchronizeBody();
                    }
                }
                if (tipoB!=GameActor::House) {
                    Creature* crB=dynamic_cast<Creature*>(evt.mGameActorB);
                    if (crB!=mPj) {
                        crB->getNode()->setPosition(crB->getPosition() - direction*0.15);
                        //crB->applyVelocity(direction.x,direction.z);          
                        Ogre::Vector3 finalPosB=crB->getPosition();        
                        crB->setHeight(mGestor->getHeightAtPosition(finalPosB));   
                        crB->synchronizeBody();
                    }
                }
        }
	return false;
}


/*
void EstadoJugando::tratarComenzarAtaque(Evt_ComenzarAtaque evt) {
        Attack* ataque=dynamic_cast<Attack*>(evento);
	Attack::ATTACK_TYPE tipoAtaque=evt.mAtaque->getType();
	if (tipoAtaque==Attack::MELEE || tipoAtaque==Attack::RANGE) {
		//si es de un solo objetivo, lanzamos un rayo
		Ogre::Vector3 direccion=evt.mAtacante->getOrientation() * Ogre::Vector3::UNIT_Z;
		Ogre::Ray rayo(evt.mAtacante->getPosition(),direccion);
		Ogre::RaySceneQuery *query=mGestor->getSceneManager()->createRayQuery(rayo);
		query->setSortByDistance(true);
		Ogre::RaySceneQueryResult& result = query->execute();
		Ogre::RaySceneQueryResult::iterator iter = result.begin();
		GameActor* actorHerido=0;
		for(iter; iter != result.end(); iter++) {
			int index;
			try {
				index=atoi((*iter).movable->getName().c_str());
				if (index>0) { //cero no es nada
					actorHerido=mGestor->getGameActorAt(index);
					if (actorHerido && actorHerido->getType()==GameActor::Player) {
						actorHerido=0;
					}
					else {
						break;//nos quedamos con el primero que no sea el propio jugador
					}
				}
			}
			catch (Exception &e) {
				//el indice no es correcto (se supone un terreno o similar)
			}
		}
		if (actorHerido) {
			Ogre::Vector3 pos1=evt.mAtacante->getPosition();
			Ogre::Vector3 pos2=actorHerido->getPosition();
			if (pos1.distance(pos2)<=evt.mAtaque->getRange()) { //si el enemigo esta al alcance del ataque
				if (tipoAtaque==Attack::MELEE) {
					//((FireAttack*)evt.mAtaque)->executeAttack(actorHerido);
					((MeleeAttack*)evt.mAtaque)->executeAttack(actorHerido);
				}
				else {
					((RangeAttack*)evt.mAtaque)->executeAttack(actorHerido);
				}
			}
			else {
				evt.mAtaque->executeAttack();
			}
		}
		else {
			//((FireAttack*)evt.mAtaque)->executeAttack(evt.mAtacante);
			if (tipoAtaque==Attack::MELEE) {
				((MeleeAttack*)evt.mAtaque)->executeAttack();
			}
			else {
				((RangeAttack*)evt.mAtaque)->executeAttack();
			}
		}
	}
	else { //el ataque es en area
		//TODO
	}
}*/