/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "GardeGames/GardeGames.h"
#include "GestorEscenario.h"
#include <Ogre.h>
#include "HumanView.h"

#include "Creature.h"
#include "StaticActor.h"
#include "IA/AIViewDeer.h"
#include "IA/AIViewWolf.h"
#include "IA/AIViewDragon.h"

#include "Evt_ComenzarAtaque.h"

#include "PHYSICS/collisionManager.h"
#include "Sound/soundFXManager.h"
#include <Ogre/SdkTrays.h>

//! Clase que representa el estado de juego en el que se está jugando una partida.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Hereda de GameState y es la clase encargada de actualizar la partida (todas las criaturas, etc.)
 *  así como de recoger diferentes eventos y tratarlos. Por ejemplo, recoge los eventos de colisión
 *  y separa a las dos criaturas que chocan; si una no es una criatura sino que es un ataque, 
 *  esta clase se encarga de quitar el ataque del juego y de dañar a la criatura contra la que ha chocado.
 *  También se encarga de otros muchos tipos de eventos, de visión, de movimiento, etc.
 *  Básicamente trata todos los eventos que requieran la interacción del escenario.
 * 
 *  La clase contiene una lista de IGameView que actualiza en cada iteración;
 *  también se encarga de decirle al controlador de físicas que se actualice, y lo mismo con el gestor de sonidos
 * 
 *  Crea la interfaz gráfica (barra de vida del personaje) y carga las criaturas mediante XML.
 * 
 */

class EstadoJugando : public GardeGames::GameState,  public GardeGames::IEventListener
{
public:
	EstadoJugando(void);
	~EstadoJugando(void);

	//heredadas de GameState
	void start(GardeGames::InputManager *inputManager);
	void stop();
	void update(long elapsedTime);
	void loadResources();
        
        void configurarOverlay();
        
        //cargar actores desde xml
        void loadActorsFromXML(const Ogre::String& fileName);
          
	//heredadas de IEventListener
	bool HandleEvent( GardeGames::Event & evento );
	virtual char const * GetName(void){return "EstadoJugando";}
        
        void inicializarLobo(Ogre::Vector3 pos,double maxSpeed);
        void inicializarCiervo(Ogre::Vector3 pos,double maxSpeed);
         void inicializarDragon(Ogre::Vector3 pos,double maxSpeed);
        
        void HandlePhysics_Body_Body_BeginEvent(Body* bodyA, Body* bodyB);
        void HandlePhysics_Attack_Body_BeginEvent(Body* bodyA, Body* bodyB);
        //void HandlePhysicsDuringEvent(GameActor* _1, GameActor* _2);
        //void HandlePhysicsEndEvent(GameActor* _1, GameActor* _2);

	Ogre::Root           *mRoot;
	Ogre::RenderWindow   *mRenderWindow;
    Ogre::Camera         *mCamera;
	/*Ogre::SceneNode		 *mCameraNode;
	Ogre::SceneNode		 *mCameraPivot;
	Ogre::SceneNode		 *mCameraGoal;
	float				 mPivotPitch;*/
    Ogre::SceneManager   *mSceneMgr;
	float				 mAlturaCamara;

	float mElapsedTime;
        
        //OVERLAY
        Ogre::OverlayManager *mGestorOverlay;
        Ogre::Overlay *mOverlay;
        Ogre::PanelOverlayElement *mLifeBar;
        Ogre::PanelOverlayElement *mBackBar;


	GardeGames::InputManager *mInputManager;
	GardeGames::GameAction *mAccionSalir;	
	GardeGames::GameAction *mAccionAlante;
	GardeGames::GameAction *mAccionAtras;
	GardeGames::GameAction *mAccionIzda;
	GardeGames::GameAction *mAccionDcha;
	GardeGames::GameAction *mAccionRatonMovido;
	GardeGames::GameAction *mAccionGolpea;
	GardeGames::GameAction *mAccionLanzaRayo;
        GardeGames::GameAction *mAccionVolumeUp;
	GardeGames::GameAction *mAccionVolumeDown;
        //acciones de transformacion
	GardeGames::GameAction *mAccionTransElfa;
	GardeGames::GameAction *mAccionTransCiervo;
	GardeGames::GameAction *mAccionTransLobo;

	HumanView *mHumanView;	
	AIViewDeer *mAIView1;
	AIViewWolf *mAIView2;
        AIViewDragon *mAIView3;
        
        std::vector<GardeGames::IGameView*> mGameViews;

	GestorEscenario* mGestor;

	//variables de la fisica (Critter y NxOgre)
	//GameActorManager* mGameActorManager;

	//actores
	Creature* mPj;
	Creature* mCiervo;        
	Creature* mLobo;
	StaticActor* mCasa1;
	StaticActor* mCasa2;
	StaticActor* mCasa3;

	//prueba
	Ogre::SceneNode* mCajaTL;
	Ogre::SceneNode* mCajaTR;
	Ogre::SceneNode* mCajaBL;
	Ogre::SceneNode* mCajaBR;
        
        CollisionManager* mCollisionManager;

private:
	//void tratarComenzarAtaque(Evt_ComenzarAtaque evt); 
};
