/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

//#include <Critter.h>
#include "Ogre.h"
#include "PHYSICS/body.h"
#include "PHYSICS/shape.h"

//! Clase que representa cualquier ente del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Modela los entes dentro del juego, ya sean estáticos, controlables, inteligentes, etc.
 *  
 *  Define los tipos de actores que hay y asocia a los entes con un nodo, una malla y un cuerpo de física
 *  Es, en definitiva, la clase madre de los elementos del juego.
 */

class GameActor {

	public:

		enum ActorType {
			Deer,
			Player,
                        Wolf,
                        Dragon,
                        Attack,
			House
		};

		GameActor(int id,Ogre::SceneNode* node, ActorType type, Shape* shape,float height);
		virtual ~GameActor(void);

		int getId();
		ActorType getType();
		Ogre::SceneNode* getNode();
		Ogre::Vector3 getPosition();
                
                void restorePosition();
                void rememberPosition();
                void synchronizeBody();
                
                void setBody(Body* body);
                Body* getBody();
                float getHeightOverFloor();

	private:                
		int mId;
		Ogre::SceneNode* mNode;
                float mHeightOverFloor;
		ActorType mType;
                Body* mBody;
                std::vector<Shape*> mShapes;
                Ogre::Vector3 mOldPosition;
                

};