// InputManager.cpp
// compile with: /EHsc /LD

#include "GameAction.h"

#include <stdexcept>

using namespace std;

namespace GardeGames
{
	GameAction::GameAction() {
        this->name = NULL;
        this->behavior = NORMAL;
        reset();
    }

	/**
        Create a new GameAction with the NORMAL behavior.
    */
	GameAction::GameAction(char* name) {
        this->name = name;
        this->behavior = NORMAL;
        reset();
    }


    /**
        Create a new GameAction with the specified behavior.
    */
    GameAction::GameAction(char* name, int behavior) {
        this->name = name;
        this->behavior = behavior;
        reset();
    }

	GameAction::~GameAction() {
	}


    /**
        Gets the name of this GameAction.
    */
    char* GameAction::getName() {
        return name;
    }


    /**
        Resets this GameAction so that it appears like it hasn't
        been pressed.
    */
    void GameAction::reset() {
        state = STATE_RELEASED;
        amount = 0;
    }


    /**
        Taps this GameAction. Same as calling press() followed
        by release().
    */
    void GameAction::tap() {
        press();
        release();
    }


    /**
        Signals that the key was pressed.
    */
    void GameAction::press() {
        press(1);
    }


    /**
        Signals that the key was pressed a specified number of
        times, or that the mouse move a spcified distance.
    */
    void GameAction::press(int amount) {
        if (state != STATE_WAITING_FOR_RELEASE) {
            this->amount+=amount;
            state = STATE_PRESSED;
        }
    }


    /**
        Signals that the key was released
    */
    void GameAction::release() {
        state = STATE_RELEASED;
    }

    /**
        Returns whether the key was pressed or not since last
        checked.
    */
    bool GameAction::isPressed() {
        return (getAmount() != 0);
    }

    /**
        For keys, this is the number of times the key was
        pressed since it was last checked.
        For mouse movement, this is the distance moved.
    */
    int GameAction::getAmount() {
        int retVal = amount;
        if (retVal != 0) {
            if (state == STATE_RELEASED) {
                amount = 0;
            }
            else if (behavior == DETECT_INITAL_PRESS_ONLY) {
                state = STATE_WAITING_FOR_RELEASE;
                amount = 0;
            }
        }
        return retVal;
    }
}