#include "GameManagerEventManaged.h"
#include <windows.h>
using namespace Ogre;

namespace GardeGames
{

	GameManagerEventManaged::GameManagerEventManaged( void ) {
		bShutdown= false ; 
	}

	GameManagerEventManaged::~GameManagerEventManaged( void ) {

		if( mInputMgr ) {
			delete mInputMgr;
			mInputMgr = 0;
		}

		if( mStateMgr ) {
			delete mStateMgr;
			mStateMgr = 0;
		}

		if( mEventMgr ) {
			delete mEventMgr;
			mEventMgr = 0;
		}
		
		if( mRoot ) {
			delete mRoot;
			mRoot = 0;
		}
	}


	void GameManagerEventManaged::startGame(std::vector<GameState*> estados,std::string estadoInicial) {

		Ogre::String mResourcesCfg;
		Ogre::String mPluginsCfg;

		#ifdef _DEBUG
			mResourcesCfg = "resources_d.cfg";
			mPluginsCfg = "plugins_d.cfg";
		#else
			mResourcesCfg = "resources.cfg";
			mPluginsCfg = "plugins.cfg";
		#endif

		mRoot = new Root(mPluginsCfg);

		// Setup and configure game
		this->setupResources(mResourcesCfg);
		if( !this->configureGame() ) {
			// If game can't be configured, throw exception and quit application
			throw Ogre::Exception( Ogre::Exception::ERR_INTERNAL_ERROR,
				"Error - Couldn't Configure Renderwindow",
				"Example - Error" );
			return;
		}


		// Setup input
		OIS::ParamList paramList;    
        size_t windowHnd = 0;
        std::ostringstream windowHndStr;

        // Get window handle
#if defined OIS_WIN32_PLATFORM
        mRenderWindow->getCustomAttribute( "WINDOW", &windowHnd );
#elif defined OIS_LINUX_PLATFORM
        mRenderWindow->getCustomAttribute( "GLXWINDOW", &windowHnd );
#endif

        // Fill parameter list
        windowHndStr << (unsigned int) windowHnd;
        paramList.insert( std::make_pair( std::string( "WINDOW" ), windowHndStr.str() ) );

        // Create inputsystem
        OIS::InputManager *mInputSystem = OIS::InputManager::createInputSystem( paramList );

        // If possible create a buffered keyboard
        OIS::Keyboard* mKeyboard = static_cast<OIS::Keyboard*>( mInputSystem->createInputObject( OIS::OISKeyboard, true ) );

        // If possible create a buffered mouse
        OIS::Mouse* mMouse = static_cast<OIS::Mouse*>( mInputSystem->createInputObject( OIS::OISMouse, true ) );


		//---------------- ZONA PROPIA -------------\\
		//creamos InputManager y asociamos teclado y raton
		mInputMgr = new InputManager();

		mInputMgr->setKeyboard(mKeyboard);
		mInputMgr->setMouse(mMouse);

		// creamos StateManager y cargamos los estados del usuario
		mStateMgr=new GameStateManager(mInputMgr);

		// creamos EventManager de forma global
		mEventMgr=new EventManager("EventManager",true);

		while (!estados.empty()) {
			estados.back()->loadResources();
			mStateMgr->addState(estados.back());
			estados.pop_back();
		}

		mStateMgr->setState(estadoInicial);

		//permite detectar Alt+Tab y cosas asi
		Ogre::WindowEventUtilities::addWindowEventListener(mRoot->getAutoCreatedWindow(),this);


		// lTimeLastFrame remembers the last time that it was checked
		// We use it to calculate the time since last frame
		unsigned long lTimeLastFrame = 0;
		unsigned long tiempoPasado = 0;
		unsigned long maxTiempo = 10;

		// Main while-loop
		while( !bShutdown ) {
			// Calculate time since last frame and remember current time for next frame
			unsigned long lTimeCurrentFrame = mRoot->getTimer()->getMilliseconds();
			unsigned long lTimeSinceLastFrame = lTimeCurrentFrame - lTimeLastFrame;
			lTimeLastFrame = lTimeCurrentFrame;
			tiempoPasado +=  lTimeSinceLastFrame;

			

			while (tiempoPasado>=maxTiempo) {
                            
                            // Update inputmanager
                            mInputMgr->capture();
                            
                            // Update current state
                            mStateMgr->update( maxTiempo/*lTimeSinceLastFrame*/ );                                                      
                            
                            mEventMgr->tick(maxTiempo/*lTimeSinceLastFrame*/);
                            tiempoPasado-=maxTiempo;
			}

			//Lee los eventos de la ventana (minimizar, maximizar, etc.)
			Ogre::WindowEventUtilities::messagePump();

			// Render next frame
                        mRoot->renderOneFrame();

			//DWORD dw=0.01;
			//Sleep(dw);

			if (mStateMgr->isDone()) {
				requestShutdown();
			}

			// Deal with platform specific issues
			//PlatformManager::getSingletonPtr()->messagePump( mRenderWindow );
		}
	}


	bool GameManagerEventManaged::configureGame( void ) {
 
		/*Ogre::RenderSystemList *renderSystems = NULL;
 
		Ogre::RenderSystemList::iterator r_it;
	 
		// hacemos este paso solo para conseguir un iterador que podemos usar con setRenderSystem. En un articulo futuro
	 
		// ahora iteramos la lista a mostrar con los sistemas de renderizado disponibles.
	 
		*renderSystems = mRoot->getAvailableRenderers();
	 
		r_it = renderSystems->begin();
	 
		mRoot->setRenderSystem(*r_it);*/

		if( !mRoot->restoreConfig() ) {
			// If there is no config file, show the configuration dialog
			if( !mRoot->showConfigDialog() ) {
				return false;
			}
		}

		mRenderWindow=mRoot->initialise(true);
                
                Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();                
	 
		// carga los plugins comunes
	 
/*	#if defined _DEBUG
	 
		mRoot->loadPlugin("Plugin_CgProgramManager_d");
	 
		mRoot->loadPlugin("Plugin_OctreeSceneManager_d");
	 
	#else
	 
		mRoot->loadPlugin("Plugin_CgProgramManager");
	 
		mRoot->loadPlugin("Plugin_OctreeSceneManager");
	 
	#endif	
	 
		// load the basic resource location(s)
	 
		Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
	 
			"resource", "FileSystem", "General");
	 
		Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
	 
			"resource/gui.zip", "Zip", "GUI");
	 
	#if defined(WIN32)
	 
		Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
	 
			"c:\\windows\\fonts", "FileSystem", "GUI");
	 
	#endif
	 
		Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
	 
		Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("GUI");*/
	 
		// establece la ventana principal; codifica algunas opciones por defecto para la presentacion
	 
		Ogre::NameValuePairList opts;
	 
		opts["resolution"] = "1024x768";
	 
		opts["fullscreen"] = "false";
	 
		opts["vsync"] = "false";
	 
	 
		// crea una ventana de renderizado con el titulo "CDK"
	 
		//mRenderWindow = mRoot->createRenderWindow("CDK", 1024, 768, false, &opts);
		
		//mRoot->createSceneManager( ST_GENERIC, "ST_GENERIC" );
		return true;
	}

 
	void GameManagerEventManaged::setupResources( Ogre::String mResourcesCfg ) {

		// Load resource paths from config file
		ConfigFile cf;
		cf.load( mResourcesCfg );

		// Go through all settings in the file
		ConfigFile::SectionIterator itSection = cf.getSectionIterator();

		String sSection, sType, sArch;
		while( itSection.hasMoreElements() ) {
			sSection = itSection.peekNextKey();

			ConfigFile::SettingsMultiMap *mapSettings = itSection.getNext();
			ConfigFile::SettingsMultiMap::iterator itSetting = mapSettings->begin();
			while( itSetting != mapSettings->end() ) {
				sType = itSetting->first;
				sArch = itSetting->second;
	                            
				ResourceGroupManager::getSingleton().addResourceLocation(
					sArch, sType, sSection );

				++itSetting;
			}
		}

	}
 
/*
	bool GameManager::configureGame( void ) {
		// Load config settings from ogre.cfg
		if( !mRoot->restoreConfig() ) {
			// If there is no config file, show the configuration dialog
			if( !mRoot->showConfigDialog() ) {
				return false;
			}
		}

		// Initialise and create a default rendering window
		mRoot->initialise(false);
		mRenderWindow = mRoot->createRenderWindow("Prueba2",800,600,false);

		// Initialise resources
		//ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

		// Create needed scenemanagers
		mRoot->createSceneManager( ST_GENERIC, "ST_GENERIC" );

		return true;
	}


	void GameManager::setupResources( void ) {
		// Load resource paths from config file
		ConfigFile cf;
		cf.load( "resources_d.cfg" );

		// Go through all settings in the file
		ConfigFile::SectionIterator itSection = cf.getSectionIterator();

		String sSection, sType, sArch;
		while( itSection.hasMoreElements() ) {
			sSection = itSection.peekNextKey();

			ConfigFile::SettingsMultiMap *mapSettings = itSection.getNext();
			ConfigFile::SettingsMultiMap::iterator itSetting = mapSettings->begin();
			while( itSetting != mapSettings->end() ) {
				sType = itSetting->first;
				sArch = itSetting->second;
	                            
				ResourceGroupManager::getSingleton().addResourceLocation(
					sArch, sType, sSection );

				++itSetting;
			}
		}
	}*/

	void GameManagerEventManaged::requestShutdown( void ) {
		bShutdown = true;
	}
	
/*	//Estos metodos deberan ser sobreescritos por las clases herederas
	std::vector<GameState*> GameManager::getEstados(){
		std::vector<GameState*> v;
		return v;
	}*/
}