/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "InputManager.h"
#include <OIS.h>


//! Clase abstracta que representa un estado del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Representa un estado en el juego, ya sea el menú principal, el de opciones, el de partida en curso, etc.
 *  Sirve para saber qué debe actualizarse y dibujarse por pantalla
 * 
 *  Ejemplo:
 *
 *  \code
 *  MiEstado.h:
 *  class MiEstado : GameState 
 *  {      
 *      public:
 *              MiEstado();
 *      //declaración de métodos abstractos
 *  }
 * 
 *  MiEstado.cpp:
 *  MiEstado::MiEstado()
 *    : GameState("MiEstado")
 *  {
 *  }
 *  //implementación de métodos abstractos
 * 
 *  \endcode
 */

namespace GardeGames {
	class GameState {
	public:
		__declspec(dllexport)	GameState( std::string nombre );
		~GameState( void ) { }	

		virtual void start(InputManager *inputManager)=0;
		virtual void stop()=0;
		virtual void update(long elapsedTime)=0;
		virtual void loadResources()=0;

		std::string getName();
		std::string checkForStateChange();
		std::string siguienteEstado;

		void keyPressed( const OIS::KeyEvent &e );
		void keyReleased( const OIS::KeyEvent &e );
			
		void mouseMoved( const OIS::MouseEvent &arg );
		void mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id );
		void mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id  );

	protected:
		//virtual void draw(Graphics2D g);	

	private:
		std::string nombre;
	};
}
