/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "GameState.h"


//! Clase que gestiona los distintos estados del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Gestiona todos los estados que existen en el juego, que los tiene guardados en una lista.
 *  Sabe qué estado es el actual, el que debe actualizarse y mostrarse por pantalla.
 *  También se encarga de cambiar de estado cuando se le solicita.
 * 
 *  En principio esta clase se autogestiona completamente,
 *  no hace falta usarla ni tocarla para nada.
 * 
 */


namespace GardeGames {
	class GameStateManager
	{
	public:
		__declspec(dllexport)	GameStateManager(InputManager *inputManager);
		__declspec(dllexport)	~GameStateManager(void);

		void addState(GameState* state);
		//std::iterator getStates();
		bool isDone();
		void setState(std::string name);
		void update(long elapsedTime);

		__declspec(dllexport)	static std::string exitGame();


	private:
		std::map<std::string,GardeGames::GameState*> gameStates;
		GameState* currentState;
		InputManager *inputManager;
		bool done;
	};
}
