/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "Event.h"


//! Clase abstracta que representa un listener del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Representa una clase que quiere recibir y procesar algún tipo de eventos.
 * 
 *  Ejemplo:
 *
 *  \code
 *  MiListener.h:
 *  class MiListener : IEventListener 
 *  {      
 *      public:
 *              MiListener();
 * 
 *              // heredadas
 *              bool HandleEvent( Event & evento );
 *  }
 * 
 *  MiListener.cpp:
 *  MiListener::MiListener()
 *  {
 *  }
 * 
 *  bool MiListener::HandleEvent( Event & evento )
 *  {
 *      //codigo para procesar eventos
 *  }
 * 
 *  \endcode
 */

namespace GardeGames {
	class IEventListener
	{
	public:

		explicit IEventListener()
		{}

		virtual ~IEventListener()
		{}

		// Returns ascii-text name for this listener, used mostly for
		// debugging

		virtual char const * GetName(void) = 0;

		// Return 'false' to indicate that this listener did NOT
		// consume the event, ( and it should continue to be
		// propogated )
		// 					
		// return 'true' to indicate that this listener consumed the
		//event, ( and it should NOT continue to be propgated )

		virtual bool HandleEvent( Event & evento ) = 0;
		//{		
			// Note: while HandleEvent() MUST be implemented in all
			// derivative classes, (as this function is pure-virtual
			// and thus the hook for IEventListener being an
			// interface definition) a base implementation is
			// provided here to make it easier to wire up do-nothing
			// stubs that can easily be wired to log the
			// unhandled-event (once logging is available)

			// HandleEvent() functioning should be kept as brief as
			// possible as multiple events will need to be evaluated
			// per-frame in many cases.
		//}
	};
}
