/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once


//! Clase abstracta que representa un elemento con comportamiento dinámico en el juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Representa todo tipo de elementos cuyo comportamiento pueda variar,
 *  ya sea por acción humana o de forma autónoma (jugador, PNJs, animales, etc.).
 * 
 *  Las vistas que heredan de esta clase suelen tener asociado un listener,
 *  de modo que así se puede determinar el comportamiento teniendo en cuenta lo que pasa alrededor.
 * 
 *  Ejemplo:
 *
 *  \code
 *  MiVista.h:
 *  class MiVista : IGameView
 *  {      
 *      public:
 *              MiVista(Actor* actor);
 *              Actor* actor;
 * 
 *              //declaración de métodos heredados
 *              void VOnUpdate( int deltaMilliseconds );
 *  }
 * 
 *  MiVista.cpp:
 *  MiVista::MiVista(Actor* actor)
 *  {
 *      this.actor = actor;
 *  }
 * 
 *  //implementación de métodos heredados (este metodo debe llamarse al actualizar el estado del juego).
 *  void VOnUpdate( int deltaMilliseconds )
 *  {
 *      //actualizar el actor y disparar eventos según corresponda
 *  }
 * 
 *  \endcode
 */

namespace GardeGames {

	class IGameView 
	{

	public:
		
		enum GameViewType
		{
			GameView_Human,
			GameView_Remote,
			GameView_AI,
			GameView_Recorder,
			GameView_Other
		};
		typedef unsigned int GameViewId;
		//virtual HRESULT VOnRestore()=0;
		//virtual void VOnRender(double fTime, float fElapsedTime)=0;
		//virtual void VOnLostDevice()=0;
		virtual GameViewType VGetType()=0;
		virtual GameViewId VGetId() const=0;
		//virtual void VOnAttach(GameViewId vid, optional<ActorId> aid)=0;

		//virtual LRESULT CALLBACK VOnMsgProc( AppMsg msg )=0;
		virtual void VOnUpdate( int deltaMilliseconds )=0;

		virtual ~IGameView() { };
	};
}
