#include "GestorEscenario.h"

#include "Creature.h"
//#include "GameActorManager.h"
#include "GameActor.h"

#include "PHYSICS/shape.h"
#include "Sound/sound3D.h"


//using namespace Ogre;

GestorEscenario* GestorEscenario::gestor;

GestorEscenario::GestorEscenario(void)
{
	mSceneManager=Ogre::Root::getSingletonPtr()->createSceneManager(Ogre::ST_EXTERIOR_FAR,"ST_EDITOR");
	//mScene=GameActorManager::getSingleton()->mScene;
	mCamera=0;
	mTerrainGroup=0;
	mTerrainGlobals=0;
	mNumElementos=0;
        
        mNumActors=1;
        mNumAttacks=0;
}

GestorEscenario::~GestorEscenario(void)
{
}

GestorEscenario *GestorEscenario::getSingletonPtr() {
	if (!gestor) {
		gestor=new GestorEscenario();
	}
	return gestor;
}

Ogre::SceneManager* GestorEscenario::getSceneManager() {
	return mSceneManager;
}

//A�ade o quita un objeto que haya sido colocado en el escenario
Ogre::SceneNode* GestorEscenario::addElement(tipoElemento tipo, Ogre::Vector3 pos){
	Ogre::Entity* ent;
	char nombre[16];
	sprintf(nombre, "Elemento%d", mNumElementos++);
	switch (tipo) {
		case OGRO:
			ent = mSceneManager->createEntity(std::string(nombre), "sinbad.mesh");
			break;
		case NINJA:
			ent = mSceneManager->createEntity(std::string(nombre), "ninja.mesh");
			break;
		case ROBOT:
			ent = mSceneManager->createEntity(std::string(nombre), "robot.mesh");
			break;
		default:
			return 0;
	}
	ent->setQueryFlags(tipo);	
	Ogre::SceneNode* nuevoObjeto = mSceneManager->getRootSceneNode()->createChildSceneNode(std::string(nombre),pos);
	nuevoObjeto->attachObject(ent);
	nuevoObjeto->scale(10,10,10);
	Elemento* e=new Elemento(tipo,pos);
	mEscenario->addElement(e);
	return nuevoObjeto;
}
void GestorEscenario::removeElement(Ogre::SceneNode* objeto){
	Elemento *e=new Elemento(1,objeto->getPosition());
	mEscenario->removeElement(e);
	mSceneManager->destroySceneNode(objeto);
}

//establece la ubicacion en la que aparecera el jugador
void GestorEscenario::setInitialPosition(Ogre::Vector3 pos) {
	mEscenario->setInitialPosition(pos);
}

//guarda, carga o crea un escenario a partir de un nombre dado
bool GestorEscenario::saveEscenario(){
	return mEscenario->saveEscenario();
}
bool GestorEscenario::loadEscenario(char* nombre){
	limpiarGestor();
	mNumElementos=0;
	mEscenario=new Escenario(nombre,nombre);
	destroyScene();
	createScene();
	if (!mEscenario->loadEscenario()) {
		return false;
	}
	return true;
}

bool GestorEscenario::createNewEscenario(char* archivo, char* nombre){
	limpiarGestor();
	mNumElementos=0;
	mEscenario=new Escenario(nombre,archivo);
	destroyScene();
	createScene();

	return true;
}

//establece el terreno (suelo) y cielo del escenario a partir de los nombres de las imagenes
bool GestorEscenario::setTerreno(std::string archivo){return true;}
bool GestorEscenario::setCielo(std::string archivo){return true;}

//cambia la ambientacion visual
void GestorEscenario::setNiebla(Ogre::FogMode tipo,Ogre::ColourValue color,float densidad,float comienzo, float fin){
	mSceneManager->setFog(tipo,color,densidad,comienzo,fin);

}
void GestorEscenario::setLuz(Ogre::ColourValue difuso, Ogre::ColourValue specular, Ogre::ColourValue ambient){
	mSceneManager->destroyAllLights();
	Ogre::Light* light = mSceneManager->createLight("Luz");
    light->setType(Ogre::Light::LT_DIRECTIONAL);
	Ogre::Vector3 lightdir(0.55, -0.3, 0.75);
    light->setDirection(lightdir);
    light->setDiffuseColour(difuso);
    light->setSpecularColour(specular);
 
    mSceneManager->setAmbientLight(ambient);

	mTerrainGlobals->setLightMapDirection(light->getDerivedDirection());
	mTerrainGlobals->setCompositeMapAmbient(mSceneManager->getAmbientLight());
	mTerrainGlobals->setCompositeMapDiffuse(light->getDiffuseColour());

	mEscenario->setLuz(light,ambient);
}


Ogre::Camera* GestorEscenario::getCamera() {
	if (!mCamera) {
		mCamera=mSceneManager->createCamera("camara");
	}
	return mCamera;
}

Ogre::SceneNode* GestorEscenario::getAttachedCameraNode() {
	if(!mCamera) {
		return 0;
	}
	Ogre::SceneNode* n=mSceneManager->getRootSceneNode()->createChildSceneNode("cameraNode");
	n->attachObject(mCamera);
	return n;
}

Ogre::RaySceneQuery* GestorEscenario::getRay() {
	return mSceneManager->createRayQuery(Ogre::Ray());
}


Ogre::SceneQueryResult GestorEscenario::getVolumeResult(Ogre::Ray topLeft, Ogre::Ray topRight,
							Ogre::Ray botLeft, Ogre::Ray botRight) {
        Ogre::PlaneBoundedVolume vol;
	vol.planes.push_back(Ogre::Plane(topLeft.getPoint(3), topRight.getPoint(3), botRight.getPoint(3)));         // front plane
	vol.planes.push_back(Ogre::Plane(topLeft.getOrigin(), topLeft.getPoint(100), topRight.getPoint(100)));         // top plane
	vol.planes.push_back(Ogre::Plane(topLeft.getOrigin(), botLeft.getPoint(100), topLeft.getPoint(100)));       // left plane
	vol.planes.push_back(Ogre::Plane(botLeft.getOrigin(), botRight.getPoint(100), botLeft.getPoint(100)));   // bottom plane
	vol.planes.push_back(Ogre::Plane(topRight.getOrigin(), topRight.getPoint(100), botRight.getPoint(100)));     // right plane

	Ogre::PlaneBoundedVolumeList volList;
	volList.push_back(vol);

	Ogre::PlaneBoundedVolumeListSceneQuery* volQuery = mSceneManager->createPlaneBoundedVolumeQuery(volList);
	return volQuery->execute();
}

Ogre::TerrainGroup::RayResult  GestorEscenario::getRayCollision(Ogre::Ray rayo) {
	return mTerrainGroup->rayIntersects(rayo);
}

float GestorEscenario::getHeightAtPosition(float x, float y, float z) {
	return mTerrainGroup->getHeightAtWorldPosition(x,y,z);
}

float GestorEscenario::getHeightAtPosition(float x, float z) {
	return getHeightAtPosition(x,0,z);
}

float GestorEscenario::getHeightAtPosition(Ogre::Vector3 &v) {
	return getHeightAtPosition(v.x,v.y,v.z);
}

bool GestorEscenario::isLoading() {
	if (mTerrainGroup->isDerivedDataUpdateInProgress()) {
		return true;
	}
	else {
		if (mTerrainsImported)
        {
            mTerrainGroup->saveAllTerrains(true);
            mTerrainsImported = false;
        }
		return false;
	}
}

void GestorEscenario::limpiarGestor(){
	mSceneManager->destroyAllEntities();
	mSceneManager->destroyAllLights();
	char nombre[16];
	while(mNumElementos>0) {
                sprintf(nombre, "Elemento%d", --mNumElementos);
		if (mSceneManager->hasSceneNode(nombre)) { //puede que ya se haya eliminado a mano (con suprimir)
			mSceneManager->destroySceneNode(nombre);
		}
	}        
}


GameActor* GestorEscenario::getGameActorAt(int index) {
	return mListGameActor[index];
}


GameActor* GestorEscenario::createGameActor(GameActor::ActorType type,Ogre::Vector3 position) 
{
    if (mNumActors>=MAX_ACTORS_PER_MAP) {
        return NULL;
    }
	GameActor* ga;
	char nombre[16];
	sprintf(nombre, "%d", mNumActors);
        position.y=getHeightAtPosition(position);
	switch (type) {
            case GameActor::Deer :
			ga=createDeer(nombre, position);
			break;
            case GameActor::Wolf :
			ga=createWolf(nombre, position);
			break;
            case GameActor::Player :
                        ga=createPlayer(nombre, position);
                        break;
              case GameActor::Dragon :
                        ga=createDragon(nombre, position);
                        break;
	}
        mListGameActor[mNumActors]=ga;
	mNumActors++;
	return ga;
}

Attack* GestorEscenario::createAttack(Attack::ATTACK_TYPE type, GameActor* owner, Ogre::Vector3 direction, float heightAttack) 
{
    if (mNumActors>=MAX_ACTORS_PER_MAP) {
        return NULL;
    }
	Attack* at;
	char nombre[16];
	sprintf(nombre, "%d", mNumActors);
	switch (type) {
            case Attack::MELEE :
                at=createMeleeAttack(nombre,owner,direction,heightAttack);
                break;
            case Attack::RANGE :
                at=createFireAttack(nombre,owner,direction,heightAttack);
                break;
	}
        mListGameActor[mNumActors]=at;
	mNumActors++;
	return at;
}

void GestorEscenario::destroyGameActor(GameActor* gameActor) 
{       
    printf("Destruyendo GameActor.\n");
    char nombre[16];
    sprintf(nombre, "%d", gameActor->getId());
    if(gameActor->getType()!=GameActor::Attack) {
        Ogre::Entity *_entity=(Ogre::Entity *)gameActor->getNode()->detachObject(nombre);
    mSceneManager->destroyEntity(_entity);
    }
    mSceneManager->getRootSceneNode()->removeChild(gameActor->getNode());
    //mSceneManager->destroySceneNode(gameActor->getNode());
    delete gameActor;
    gameActor=0;
}

GameActor* GestorEscenario::createPlayer(char* nombre, Ogre::Vector3 position) {
	
   Ogre::Entity *cayado=mSceneManager->createEntity("cayado", "cayado.mesh");
    
   
    // Creamos el SceneNode
    Ogre::SceneNode *_nodo=mSceneManager->getRootSceneNode()->createChildSceneNode(nombre,position);
    // Creamos la entidad
    Ogre::Entity *_entity = mSceneManager->createEntity(nombre, "Elfa.mesh");
    _entity->attachObjectToBone("Bip01 R Hand", cayado);
     
    
    
    // Adjuntamos la entidad al nodo
    _nodo->attachObject(_entity);
    
    
    
    
    _nodo->scale(0.1,0.1,0.1);
    
    Shape* shape=new Sphere("playerShape",Ogre::Vector3(0, 0, 0),4);
    
    std::vector<Ogre::String> animaciones;
    animaciones.assign(Creature::NUM_STATES,"");
    animaciones.at(Creature::IDLE)=Ogre::String("Idle");
    animaciones.at(Creature::RUN)=Ogre::String("Walk");
    animaciones.at(Creature::ATTACK1)=Ogre::String("AttackMelee");
    animaciones.at(Creature::ATTACK2)=Ogre::String("AttackMelee");
    animaciones.at(Creature::DIE)=Ogre::String("Die");
    
    std::vector<Sound3D*> sonidos;
    //Sound3D* auxSound = new Sound3D("player_footsteps.wav");
    sonidos.assign(Creature::NUM_STATES,0);
    sonidos.at(Creature::RUN) = new Sound3D("player_footsteps.wav");
    sonidos.at(Creature::ATTACK1) = new Sound3D("goblinEnter.wav");
    sonidos.at(Creature::ATTACK2) = new Sound3D("goblinEnter.wav");
    sonidos.at(Creature::DIE) = new Sound3D("goblinEnter.wav");
        
    float speed=127;
    float strength=100;
    float toughness=100;
    float alturaSuelo=-0.4;
    
    return new Creature(atoi(nombre),_nodo,_entity,GameActor::Player,shape,animaciones,sonidos,speed,strength,toughness,Attack::MELEE,Attack::RANGE,alturaSuelo);
}

GameActor* GestorEscenario::createDeer(char* nombre, Ogre::Vector3 position) {
	
    // Creamos el SceneNode
    Ogre::SceneNode *_nodo=mSceneManager->getRootSceneNode()->createChildSceneNode(nombre,position);
    // Creamos la entidad
    Ogre::Entity *_entity = mSceneManager->createEntity(nombre, "ciervo.mesh");
    // Adjuntamos la entidad al nodo
    _nodo->attachObject(_entity);
    _nodo->scale(0.16,0.16,0.16);

    
    Shape* shape=new Sphere("deerShape",Ogre::Vector3(0, 0, 0),4);
    
    std::vector<Ogre::String> animaciones;
    animaciones.assign(Creature::NUM_STATES,"");
    animaciones.at(Creature::IDLE)=Ogre::String("Idle");
    animaciones.at(Creature::RUN)=Ogre::String("Walk");
    animaciones.at(Creature::DIE)=Ogre::String("Die");
    
    std::vector<Sound3D*> sonidos;
    sonidos.assign(Creature::NUM_STATES,0);
    sonidos.at(Creature::RUN) = new Sound3D("player_footsteps.wav");
    sonidos.at(Creature::ATTACK1) = new Sound3D("goblinEnter.wav");
    
    float speed=127;
    float strength=100;
    float toughness=100;
    
    return new Creature(atoi(nombre),_nodo,_entity,GameActor::Deer, shape, animaciones,sonidos,speed,strength,toughness,Attack::NONE,Attack::NONE);	
}

GameActor* GestorEscenario::createWolf(char* nombre, Ogre::Vector3 position) {
	
    // Creamos el SceneNode
    Ogre::SceneNode *_nodo=mSceneManager->getRootSceneNode()->createChildSceneNode(nombre,position);
    // Creamos la entidad
    Ogre::Entity *_entity = mSceneManager->createEntity(nombre, "lobo.mesh");
    // Adjuntamos la entidad al nodo
    _nodo->attachObject(_entity);
    _nodo->scale(0.16,0.16,0.16);
    
    Shape* shape=new Sphere("wolfShape",Ogre::Vector3(0, 0, 0),6);
    
    std::vector<Ogre::String> animaciones;
    animaciones.assign(Creature::NUM_STATES,"");
    animaciones.at(Creature::IDLE)=Ogre::String("Walk");
    animaciones.at(Creature::RUN)=Ogre::String("Walk");
    /*animaciones.at(Creature::DIE)=Ogre::String("morir");
    animaciones.at(Creature::ATTACK1)=Ogre::String("golpeo");
    animaciones.at(Creature::ATTACK2)=Ogre::String("golpeo");*/
    
    std::vector<Sound3D*> sonidos;
    sonidos.assign(Creature::NUM_STATES,0);
    sonidos.at(Creature::RUN) = new Sound3D("player_footsteps.wav");
    sonidos.at(Creature::ATTACK1) = new Sound3D("howl.wav");
    
    float speed=127;
    float strength=100;
    float toughness=100;
    return new Creature(atoi(nombre),_nodo,_entity,GameActor::Wolf, shape, animaciones,sonidos,speed,strength,toughness,Attack::MELEE,Attack::NONE); 	
}

GameActor* GestorEscenario::createDragon(char* nombre, Ogre::Vector3 position) {
	
    // Creamos el SceneNode
    Ogre::SceneNode *_nodo=mSceneManager->getRootSceneNode()->createChildSceneNode(nombre,position);
    // Creamos la entidad
    Ogre::Entity *_entity = mSceneManager->createEntity(nombre, "dragon4.mesh");
    // Adjuntamos la entidad al nodo
    _nodo->attachObject(_entity);
    _nodo->scale(0.15,0.15,0.15);
    
    Shape* shape=new Sphere("dragonShape",Ogre::Vector3(0, 0, 0),16);
    
    std::vector<Ogre::String> animaciones;
    animaciones.assign(Creature::NUM_STATES,"");
    animaciones.at(Creature::IDLE)=Ogre::String("static");
    animaciones.at(Creature::RUN)=Ogre::String("walk");
    animaciones.at(Creature::DIE)=Ogre::String("die bitch");
    animaciones.at(Creature::ATTACK1)=Ogre::String("miuuuura");
    animaciones.at(Creature::ATTACK2)=Ogre::String("bite");
    
    std::vector<Sound3D*> sonidos;
    sonidos.assign(Creature::NUM_STATES,0);
    sonidos.at(Creature::RUN) = new Sound3D("player_footsteps.wav");
    sonidos.at(Creature::ATTACK1) = new Sound3D("DragonRoaring.wav");
    
    float speed=127;
    float strength=100;
    float toughness=100;
    return new Creature(atoi(nombre),_nodo,_entity,GameActor::Dragon, shape, animaciones,sonidos,speed,strength,toughness,Attack::MELEE,Attack::NONE,0); 	
}




Ogre::Billboard* GestorEscenario::createLifeBar(GameActor* gameActor) {
        
    // Barra de vida
    Ogre::BillboardSet* _bbSetLife = mSceneManager->createBillboardSet();
    Ogre::Billboard* _lifeBar = _bbSetLife->createBillboard(Ogre::Vector3(0,0,0));  
    _lifeBar->setDimensions(10, 1);
    _lifeBar->setTexcoordRect(0,0,10,1);
    
    Ogre::SceneNode* _lifeNode = gameActor->getNode()->createChildSceneNode();

    _lifeNode->attachObject(_bbSetLife);
    _lifeNode->setPosition(Ogre::Vector3(0,6,0));
    
    return _lifeBar;
}

void GestorEscenario::destroyLifeBar(GameActor* gameActor) {
        
    printf("Obteniendo LifeBar.\n");
    Ogre::SceneNode* _lifeNode=(Ogre::SceneNode*)gameActor->getNode()->getChild(0);
    Ogre::BillboardSet* _bbSetLife=(Ogre::BillboardSet*)_lifeNode->getAttachedObject(0);
    
    printf("Destruyendo LifeBar.\n");
    _lifeNode->detachObject(_bbSetLife);
    mSceneManager->destroyBillboardSet(_bbSetLife);
    _lifeNode->getParent()->removeChild(_lifeNode);
    mSceneManager->destroySceneNode(_lifeNode);
}

Attack* GestorEscenario::createMeleeAttack(char* nombre, GameActor* owner, Ogre::Vector3 direction, float heightAttack) {
    Ogre::SceneNode* node = GestorEscenario::getSingletonPtr()->getSceneManager()->getRootSceneNode()->createChildSceneNode(nombre, owner->getPosition());

    //node->setOrientation(orientation);
    node->setOrientation(owner->getNode()->getOrientation());
    
    Shape* shape=new Sphere("attackShape",Ogre::Vector3(0, 0, 0),2);
    
    Attack* meleeAttack=new Attack(atoi(nombre),Attack::MELEE,15,100,0,node,shape,direction);
    meleeAttack->setOwner(owner);
        Ogre::Vector3 posi= meleeAttack->getNode()->getPosition();
    posi.y+=heightAttack;
    
   
    meleeAttack->getNode()->setPosition(posi);
    meleeAttack->advance(2);
    return meleeAttack;
}

Attack* GestorEscenario::createFireAttack(char* nombre, GameActor* owner, Ogre::Vector3 direction, float heightAttack) {
    Ogre::ParticleSystem* ps = GestorEscenario::getSingletonPtr()->getSceneManager()->createParticleSystem(nombre, "FireRain");
    ps->fastForward(5);	
    Ogre::SceneNode* node = GestorEscenario::getSingletonPtr()->getSceneManager()->getRootSceneNode()->createChildSceneNode(nombre, owner->getPosition());
    node->attachObject(ps);
    node->scale(0.01,0.01,0.01);
    
    //node->setOrientation(orientation);
    node->setOrientation(owner->getNode()->getOrientation());
    
    Shape* shape=new Sphere("attackShape",Ogre::Vector3(0, 0, 0),3);
    
    Attack* fireAttack=new Attack(atoi(nombre),Attack::RANGE,1000,100,0,node,shape,direction);
    fireAttack->setOwner(owner);
    
    Ogre::Vector3 posi= fireAttack->getNode()->getPosition();
    posi.y+=heightAttack;
    
   
    fireAttack->getNode()->setPosition(posi);
    fireAttack->advance(2);
    return fireAttack;
}


//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//                              CREACION DEL TERRENO                            //
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//


void GestorEscenario::createTerrain(Ogre::Light *light,std::string nombreImagen) {
	mTerrainGroup = OGRE_NEW Ogre::TerrainGroup(mSceneManager, Ogre::Terrain::ALIGN_X_Z, 100, 100);
	mTerrainGroup->setFilenameConvention(Ogre::String("BasicTutorial32Terrain"), Ogre::String("dat"));
	mTerrainGroup->setOrigin(Ogre::Vector3::ZERO);
 
	configureTerrainDefaults(light);
 
        for (long x = 0; x <= 0; ++x)
                for (long y = 0; y <= 0; ++y)
			defineTerrain(x, y, nombreImagen);
 
	// sync load since we want everything in place when we start
	mTerrainGroup->loadAllTerrains(true);
 
	if (mTerrainsImported) {
		Ogre::TerrainGroup::TerrainIterator ti = mTerrainGroup->getTerrainIterator();
		while(ti.hasMoreElements())
		{
			Ogre::Terrain* t = ti.getNext()->instance;
			initBlendMaps(t);
		}
	}
       
}

void GestorEscenario::initBlendMaps(Ogre::Terrain* terrain) {
    Ogre::TerrainLayerBlendMap* blendMap0 = terrain->getLayerBlendMap(1);
    Ogre::TerrainLayerBlendMap* blendMap1 = terrain->getLayerBlendMap(2);
    Ogre::Real minHeight0 = 70;
    Ogre::Real fadeDist0 = 40;
    Ogre::Real minHeight1 = 70;
    Ogre::Real fadeDist1 = 15;
    float* pBlend1 = blendMap1->getBlendPointer();
    for (Ogre::uint16 y = 0; y < terrain->getLayerBlendMapSize(); ++y)
    {
        for (Ogre::uint16 x = 0; x < terrain->getLayerBlendMapSize(); ++x)
        {
            Ogre::Real tx, ty;
 
            blendMap0->convertImageToTerrainSpace(x, y, &tx, &ty);
            Ogre::Real height = terrain->getHeightAtTerrainPosition(tx, ty);
            Ogre::Real val = (height - minHeight0) / fadeDist0;
            val = Ogre::Math::Clamp(val, (Ogre::Real)0, (Ogre::Real)1);
 
            val = (height - minHeight1) / fadeDist1;
            val = Ogre::Math::Clamp(val, (Ogre::Real)0, (Ogre::Real)1);
            *pBlend1++ = val;
        }
    }
    blendMap0->dirty();
    blendMap1->dirty();
    blendMap0->update();
    blendMap1->update();
}

void GestorEscenario::defineTerrain(long x, long y,std::string imagen) {
    Ogre::String filename = mTerrainGroup->generateFilename(x, y);
    if (Ogre::ResourceGroupManager::getSingleton().resourceExists(mTerrainGroup->getResourceGroup(), filename))
    {
        mTerrainGroup->defineTerrain(x, y);
    }
    else
    {
        Ogre::Image img = getTerrainImage(x % 2 != 0, y % 2 != 0, imagen);
        mTerrainGroup->defineTerrain(x, y, &img);
	mTerrainsImported=true;
    }
}

void GestorEscenario::configureTerrainDefaults(Ogre::Light* light) {
    // Configure global
    mTerrainGlobals->setMaxPixelError(1);
    // testing composite map
    mTerrainGlobals->setCompositeMapDistance(3000);
    // Important to set these so that the terrain knows what to use for derived (non-realtime) data
    mTerrainGlobals->setLightMapDirection(light->getDerivedDirection());
    mTerrainGlobals->setCompositeMapAmbient(mSceneManager->getAmbientLight());
    mTerrainGlobals->setCompositeMapDiffuse(light->getDiffuseColour());
 
    // Configure default import settings for if we use imported image
    Ogre::Terrain::ImportData& defaultimp = mTerrainGroup->getDefaultImportSettings();
    defaultimp.terrainSize = 257;
    defaultimp.worldSize = 257.0f;
    defaultimp.inputScale = 1;
    defaultimp.minBatchSize = 33;
    defaultimp.maxBatchSize = 65;
    // textures
    defaultimp.layerList.resize(3);
    defaultimp.layerList[0].worldSize = 100;
    defaultimp.layerList[0].textureNames.push_back("dirt_grayrocky_diffusespecular.dds");
    defaultimp.layerList[0].textureNames.push_back("dirt_grayrocky_normalheight.dds");
    defaultimp.layerList[1].worldSize = 30;
    defaultimp.layerList[1].textureNames.push_back("grass_green-01_diffusespecular.dds");
    defaultimp.layerList[1].textureNames.push_back("grass_green-01_normalheight.dds");
    defaultimp.layerList[2].worldSize = 200;
    defaultimp.layerList[2].textureNames.push_back("growth_weirdfungus-03_diffusespecular.dds");
    defaultimp.layerList[2].textureNames.push_back("growth_weirdfungus-03_normalheight.dds");
}

Ogre::Image GestorEscenario::getTerrainImage(bool flipX, bool flipY, std::string nombreImagen) {
    Ogre::Image img;
    img.load("terrain.png", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
    if (flipX)
        img.flipAroundY();
    if (flipY)
        img.flipAroundX();
	
    return img;
}

void GestorEscenario::setupHeightFields() {
    Ogre::TerrainGroup::TerrainIterator ti = mTerrainGroup->getTerrainIterator();
    while(ti.hasMoreElements())
    { 
        Ogre::Terrain* terrain = ti.getNext()->instance;    
    }                
}

//----------------------------------------------------------------------------------------------------

void GestorEscenario::createScene() {
    mSceneManager->setShadowTechnique(Ogre::SHADOWTYPE_TEXTURE_MODULATIVE_INTEGRATED);
 
    Ogre::MaterialManager::getSingleton().setDefaultTextureFiltering(Ogre::TFO_ANISOTROPIC);
    Ogre::MaterialManager::getSingleton().setDefaultAnisotropy(7);
 
    Ogre::Vector3 lightdir(0.55, -0.3, 0.75);
    lightdir.normalise();
 
    Ogre::Light* light = mSceneManager->createLight("tstLight");
    light->setType(Ogre::Light::LT_DIRECTIONAL);
    light->setDirection(lightdir);
    light->setDiffuseColour(Ogre::ColourValue::White);
    light->setSpecularColour(Ogre::ColourValue(0.4, 0.4, 0.4));
    light->setCastShadows(true);
 
    mSceneManager->setAmbientLight(Ogre::ColourValue(0.2, 0.2, 0.2));
 
    mTerrainGlobals = OGRE_NEW Ogre::TerrainGlobalOptions();
 
    mTerrainGroup = OGRE_NEW Ogre::TerrainGroup(mSceneManager, Ogre::Terrain::ALIGN_X_Z, 513, 12000.0f);
    mTerrainGroup->setFilenameConvention(Ogre::String("BasicTutorial32Terrain"), Ogre::String("dat"));
    //mTerrainGroup->setFilenameConvention(Ogre::String("BasicTutorial3Terrain"), Ogre::String("dat"));
    mTerrainGroup->setOrigin(Ogre::Vector3::ZERO);
 
    configureTerrainDefaults(light);
 
    for (long x = 0; x <= 0; ++x)
        for (long y = 0; y <= 0; ++y)
            defineTerrain(x, y,"terrain.png");
 
    // sync load since we want everything in place when we start
    mTerrainGroup->loadAllTerrains(true);
 
    if (mTerrainsImported)
    {
        Ogre::TerrainGroup::TerrainIterator ti = mTerrainGroup->getTerrainIterator();
        while(ti.hasMoreElements())
        {
            Ogre::Terrain* t = ti.getNext()->instance;
            initBlendMaps(t);
        }
    }
    
    //mWindow->getViewport(0)->setBackgroundColour(fadeColour);
    Ogre::Plane plane;
    plane.d = 100;
    plane.normal = Ogre::Vector3::NEGATIVE_UNIT_Y; 
    mSceneManager->setSkyPlane(true, plane, "Examples/CloudySky", 500, 20, true, 0.5, 150, 150);


    Ogre::ColourValue fadeColour(0.75, 0.84, 1);
    //mEscenario->setNiebla(Ogre::FOG_LINEAR, fadeColour, 0.0, 10, 1200);
    //mEscenario->setLuz(light,Ogre::ColourValue(0.2, 0.2, 0.2));
    setNiebla(Ogre::FOG_LINEAR, fadeColour, 0.0, 10, 1200);
    setLuz(Ogre::ColourValue::White,Ogre::ColourValue(0.2, 0.2, 0.2),Ogre::ColourValue(0.2, 0.2, 0.2));
}


void GestorEscenario::destroyScene() {
    if (mTerrainGroup)
        OGRE_DELETE mTerrainGroup;
    if (mTerrainGlobals)
        OGRE_DELETE mTerrainGlobals;
}