/* 
 * File:   GestorSonido.cpp
 * Author: Miguel
 * 
 * Created on 20 de enero de 2012, 13:05
 */

#include "GestorSonido.h"
#include "Creature.h"
#include "BackSound.h"

#define MAX_VOLUME 255

GestorSonido* GestorSonido::gestor;

GestorSonido::GestorSonido() {
    mSoundFXManager = new SoundFXManager();
    initialiseSDL();   
    mVolume=MAX_VOLUME/2;
    mMusic=0;
}

GestorSonido *GestorSonido::getSingletonPtr() {
	if (!gestor) {
		gestor=new GestorSonido();
	}
	return gestor;
}

bool GestorSonido::initialiseSDL() {
    if (SDL_Init(SDL_INIT_AUDIO) < 0)
        return false;

    atexit(SDL_Quit);

    if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT,MIX_DEFAULT_CHANNELS, 4096) < 0){
        return false;
    }

    Mix_AllocateChannels(32);

    atexit(Mix_CloseAudio);

    return true;
}

void GestorSonido::playSound(Ogre::SceneNode* listener, Ogre::SceneNode* source, Sound3D* sonido, bool loop, std::vector<Sound3D*> sonidos) {
    //if (!sonidos.empty()) {
    //    stopSounds(sonidos);
    //}
    //if (sonido->mMyChannel!=-1) { // se está reproduciendo
    //    return;
    //}
    if (sonido->mIsPlaying) {
        return;
    }
    sonido->setListenerNode(listener);
    sonido->setSourceNode(source);
    int n= loop ? 1000 : 0;
    sonido->play(n);
}

void GestorSonido::stopSounds(std::vector<Sound3D*> sonidos) {
    int i=0;
    Sound3D* aux=0;
    while (i<sonidos.size()) {
        aux=sonidos.at(i);
        if (aux && aux->mIsPlaying) {
            aux->stop();
        }
        i++;
    }
}

void GestorSonido::update(long elapsedTime) {
    Sound3D::update3DSound();
}

void GestorSonido::volumeUp(){       
    mVolume++;
    if (mVolume>MAX_VOLUME) { 
        mVolume=MAX_VOLUME;
    }
    mMusic->mVolume=mVolume;
}


void GestorSonido::volumeDown(){
    mVolume--;
    if (mVolume<0) { 
        mVolume=0;
    }
    mMusic->mVolume=mVolume;
}

void GestorSonido::setBackMusic(Ogre::String name) {
    if (mMusic)
        delete mMusic;
    
    mMusic=new BackSound(name);
    mMusic->mVolume=mVolume;
    playSound(0,0,mMusic,true);
}

GestorSonido::~GestorSonido() {
}

