/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <SDL/SDL_mixer.h>
#include <SDL/SDL.h>
#include <boost/unordered_map.hpp>
#include "Sound/SoundFXManager.h"
#include "BackSound.h"

//! Clase que gestiona el sonido del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Gestiona todos los canales de audio, cada uno con su propio volumen y estéreo,
 *  y reproduce y controla la música de fondo.
 *  
 *  Encapsula la clase SoundFXManager y
 *  amplía su funcionalidad permitiendo controlar el volumen de la música de fondo
 */

class GestorSonido {
public:
    
    GestorSonido();
    ~GestorSonido();
    
    static GestorSonido *getSingletonPtr();
    
    void playSound(Ogre::SceneNode* listener, Ogre::SceneNode* source, Sound3D* sonido, bool loop, std::vector<Sound3D*> sonidos=0);
    void stopSounds(std::vector<Sound3D*> sonidos);    
    
    void update(long elapsedTime);
    
    void setBackMusic(Ogre::String name);
    void volumeUp();
    void volumeDown();
    
private:
    static GestorSonido *gestor;
    SoundFXManager* mSoundFXManager;    
    
    BackSound* mMusic;
    int mVolume;
    
    bool initialiseSDL();

};

