 #ifndef STATEMACHINE_H
#define STATEMACHINE_H

/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Héctor Agustín Lana (C) 2012
 * <hectoragustin89@hotmail.com
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <string>

#include "State.h"


/**
 *  @author Hector Agustin Lana
 *  @date 14-01-2012
 *
 *  Esta clase modela una maquina de estados
 *  
 */

template <class entity_type>
class StateMachine
{
private:

  //puntero al que tiene la maquina
  entity_type*          m_pOwner;

  State<entity_type>*   m_pCurrentState;
  
  //estado previo
  State<entity_type>*   m_pPreviousState;

  //estado global
  State<entity_type>*   m_pGlobalState;
  

public:

  StateMachine(entity_type* owner):m_pOwner(owner),
                                   m_pCurrentState(NULL),
                                   m_pPreviousState(NULL),
                                   m_pGlobalState(NULL)
  {}

  virtual ~StateMachine(){}

  //cambiar estados
  void SetCurrentState(State<entity_type>* s){m_pCurrentState = s;}
  void SetGlobalState(State<entity_type>* s) {m_pGlobalState = s;}
  void SetPreviousState(State<entity_type>* s){m_pPreviousState = s;}
  
  //actualizar la maquina
  void  Update()const
  {
	  
    
    if(m_pGlobalState)   m_pGlobalState->Execute(m_pOwner);

   
    if (m_pCurrentState) m_pCurrentState->Execute(m_pOwner); 
  }


  //change to a new state
  void  ChangeState(State<entity_type>* pNewState)
  {
    assert(pNewState && "<StateMachine::ChangeState>:trying to assign null state to current");

    //keep a record of the previous state
    m_pPreviousState = m_pCurrentState;

    //call the exit method of the existing state
    m_pCurrentState->Exit(m_pOwner);

    //change state to the new state
    m_pCurrentState = pNewState;

    //call the entry method of the new state
    m_pCurrentState->Enter(m_pOwner);
  }

 
  void  RevertToPreviousState()
  {
    ChangeState(m_pPreviousState);
  }


  bool  isInState(const State<entity_type>& st)const
  {
    if (typeid(*m_pCurrentState) == typeid(st)) return true;
    return false;
  }

  State<entity_type>*  CurrentState()  const{return m_pCurrentState;}
  State<entity_type>*  GlobalState()   const{return m_pGlobalState;}
  State<entity_type>*  PreviousState() const{return m_pPreviousState;}


};




#endif


