/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Héctor Agustín Lana (C) 2012
 * <hectoragustin89@hotmail.com
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SteeringBehaviorsS_H
#define SteeringBehaviorsS_H
#pragma warning (disable:4786)

#include "Ogre.h"
#include "utils.h"
#include "AIView.h"
#include "navigationMesh.h"

//---------------------------- class details -----------------------------


/**
 *  @author Hector Agustin Lana
 *  @date 14-01-2012
 *
 *  Estas clases modela algunos comportamientos de movimiento
 *  
 */

class SteeringBehaviors
{

protected:




 double obstacleAvoidanceWeight;
 double priorityThreshold;
 
 double WanderRad;
 //distancia a la que el circulo de wander es proyecta en frente del agente
 double WanderDist;
 //maxima cantidad de desplazamiento en cada frame a lo largo del circulo
 double WanderJitterPerSec;

  
  AIView* mAiView;
 
 //posicion del target en el algoritmo wander
  Ogre::Vector2    m_vWanderTarget; 




public:

  SteeringBehaviors(AIView*  aiView);

  ~SteeringBehaviors();

  
 bool bSeek;
 bool bArrive;
 bool bFlee;
 bool bWander;
 bool bObstacleAvoidance;
 bool bFollowPath;

  void seekOn();
  void seekOff();
  bool isSeek();
  void arriveOn();
  void arriveOff();
  bool isArrive();
  void fleeOn();
  void fleeOff();
  bool isFlee();
  void followPathOn();
  void followPathOff();
  bool isfollowPath();
  void wanderOn();
  void wanderOff();
  bool isWander();
  void obstacleAvoidanceOn();
  void obstacleAvoidanceOff();
  bool isObstacleAvoidance();

  
  Ogre::Vector2 Seek(Ogre::Vector2 target);
  Ogre::Vector2  Wander();
  Ogre::Vector2 Evade(Ogre::Vector2 target);
  Ogre::Vector2 ObstacleAvoidance(Ogre::Vector2 obstaclePosition,double obstacleRadius);

  Ogre::Vector2 WeightedSumAlgorithm();
  
  Ogre::Vector2  SteeringBehaviors::CalculateSteeringForce();
  Ogre::Vector2 PrioritiesAlgorithm();
  Ogre::Vector2 FollowPath(navigationMesh::PointPath* path);
  
private:
    Ogre::Vector3 findTargetInPath(navigationMesh::PointPath* mPath);

};
#endif