/*
 * This file is part of SionTower.
 *
 * 
 * David Saltares Márquez (C) 2011
 * <david.saltares@gmail.com>
 *
 * 
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *  @file sound3D.cpp
 *  @author David Saltares Márquez
 *  @date 29-12-2010
 */

#include <iostream>
#include <utility>

#include <SDL/SDL_mixer.h>

#include "sound3D.h"
#include "soundFXManager.h"

using std::cout;
using std::endl;

std::vector<std::pair<int, Sound3D*> >Sound3D::_sounds;
std::vector<int> Sound3D::_channels(SoundFXManager::getAvailableChannels(), -1);
int Sound3D::_numUsedChannels = 0;
int Sound3D::mNumSounds = 0;

Sound3D::Sound3D(const Ogre::String& name,
                 const Ogre::String& group,
                 Ogre::SceneNode* listenerNode,
                 Ogre::SceneNode* sourceNode): _listenerNode(listenerNode), _sourceNode(sourceNode) {
    // Cargar el recurso
    _soundFXPtr = SoundFXManager::getSingleton().load(name, group);
    mIsPlaying=false;
    mMyChannel=-1;
    mMyNumSound=mNumSounds;
    mNumSounds++;
}

Sound3D::Sound3D(SoundFXPtr s,
                 Ogre::SceneNode* listenerNode,
                 Ogre::SceneNode* sourceNode): _soundFXPtr(s), _listenerNode(listenerNode), _sourceNode(sourceNode) 
{
    mIsPlaying=false;
    mMyChannel=-1;
}

Sound3D::~Sound3D() {}

void Sound3D::play(int loop) {
    mIsPlaying=true;
    if (mMyChannel!=-1) { //estaba pausado
        return;
    }
    // Reproducimos el sonido
    int channel = _soundFXPtr->play(loop);

    // Registramos el sonido junto con su canal para el callBack
    _sounds.push_back(std::make_pair(channel, this));
    _channels[channel] = _numUsedChannels;
    ++_numUsedChannels;
    
    mMyChannel=channel;
    //printf("--generando sonido en canal %d\n",channel);    
}

void Sound3D::stop() {
    // detenemos el sonido
    //channelFinished(mMyChannel);
    Mix_HaltChannel(mMyChannel);
    mIsPlaying=false;
    //mMyChannel=-1;
}

void Sound3D::update3DSound() {
    std::vector<std::pair<int, Sound3D*> >::iterator i;

    for (i = _sounds.begin(); i != _sounds.end(); ++i) {
        if (i->second->mIsPlaying && i->second->mMyChannel>-1) {
                i->second->update(i->first);
                // printf(">>>>>>Update sonido %d en canal %d.\n",i->second->mMyNumSound,i->second->mMyChannel);
        }
    }
}

void Sound3D::update(int channel) {
    if (_listenerNode && _sourceNode) {
        // Posiciones
        Ogre::Vector3 pE = _listenerNode->getPosition();
        Ogre::Vector3 pF = _sourceNode->getPosition();

        // Calculamos la distancia adaptada a SDL_Mixer
        Ogre::Real d = pE.distance(pF);
        d=d * 255 / 1000;

        // Aplanamos las posiciones.
        pE.y = pF.y = 0;

        // Vector entre E y F para poder calcular el ángulo
        Ogre::Vector3 FE = pF - pE;

        // Orientación del nodo
        Ogre::Quaternion q = _listenerNode->getOrientation();

        // Vector dirección del nodo
        Ogre::Vector3 direction = q * Ogre::Vector3::UNIT_Z;

        // Ángulo entre la dirección del nodo y el vector hasta la fuente
        Ogre::Radian angle = direction.angleBetween(FE);

        // Aplicamos el efecto estéreo/volumen en función del ángulo/distancia
        Mix_SetPosition(channel, angle.valueDegrees(), d);
    } else {
        // No tenemos o fuente o escucha.
        // Volumen al máximo de ese canal y ángulo a 0
        Mix_SetPosition(channel, 0, 0);
    }
   
}

const Ogre::SceneNode* Sound3D::getListenerNode() const {
    return _listenerNode;
}

const Ogre::SceneNode* Sound3D::getSourceNode() const {
    return _sourceNode;
}

void Sound3D::setListenerNode(const Ogre::SceneNode* listenerNode) {
    _listenerNode = listenerNode;
}

void Sound3D::setSourceNode(const Ogre::SceneNode* sourceNode) {
    _sourceNode = sourceNode;
}

void channelFinished(int channel) {
    Sound3D::channelFinished(channel);
}

void Sound3D::channelFinished(int channel) {
    // Si el canal estaba siendo utilizado por un efecto 3D
    if (channel != -1) {        
        //printf("--finalizando sonido en canal %d\n",channel);
        // Borramos el sonido3D del vector de sonidos en reproducción 
        if (_channels[channel]<_sounds.size()) {
            std::pair<int, Sound3D*> aux=_sounds.at(_channels[channel]);
            aux.second->mIsPlaying=false;
            aux.second->mMyChannel=-1;
        }
        //mIsPlaying=false;
        _sounds.erase(_sounds.begin() + _channels[channel]);
        //_channels[channel] = -1;
        --_numUsedChannels;
    }
}

